@@ -26,6 +26,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include <QString>
 #include <QVector>
 
+#include "Utils.h"
+
 RELAYTOOL_MAD;
 
 // Always put me below _all_ includes, this is needed
@@ -165,7 +167,7 @@ bool K3bMad::open(const QString& filename)
 	m_inputFile.setFileName(filename);
 	
 	if (!m_inputFile.open(QIODevice::ReadOnly)) {
-		PERROR("could not open file %s", m_inputFile.fileName().toUtf8().data());
+		PERROR("could not open file %s", QS_C(filename));
 		return false;
 	}
 	
@@ -275,7 +277,7 @@ bool K3bMad::skipTag()
 		
 		// skip the id3 tag
 		if (!m_inputFile.seek(offset)) {
-			PERROR("Couldn't seek to %u in %s", offset, m_inputFile.fileName().toUtf8().data());
+			PERROR("Couldn't seek to %u in %s", offset, QS_C(m_inputFile.fileName()));
 			return false;
 		}
 	}
