@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2007 Remon Sijrier 
+Copyright (C) 2007-2008 Remon Sijrier 
 
 This file is part of Traverso
 
@@ -22,7 +22,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "TraversoCommands.h"
 
 #include <libtraversocore.h>
-#include <libtraversosongcanvas.h>
+#include <libtraversosheetcanvas.h>
 #include <commands.h>
 
 
@@ -30,6 +30,71 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 // in case we run with memory leak detection enabled!
 #include "Debugger.h"
 
+/**
+ *	\class TraversoCommands
+	\brief The Traverso CommandPlugin class which 'implements' many of the default Commands
+	
+	With this plugin, the InputEngine is able to dispatch key actions by directly 
+	asking this Plugin for the needed Command object.
+
+	Dispatching key events to a specific object to create a Command object is accomplished by creating 
+	entries in the keymap.xml file, which has the following syntax:
+
+	\code 
+	<Object objectname="" mousehint="" modes="" sortorder="" pluginname="" commandname="" arguments="" />
+	\endcode 
+	
+	\code 
+	objectname: 	The class name of the object that we want to dispatch the key action on
+	mousehint 	Used to set the mouse cursor. Use "LR" for horizontal suggestion, "UD" for vertical suggestion, or "RLUD" for omnidirectional suggestion.
+	modes:		Modes for which this command can be used, like All, Edit or Effects. Modes are to be defined in the keymap itself!
+	sortorder:	The order of where the key action should show up in the ContextMenu of 'objectname'
+	plugniname:	The plugin to ask the Command for, the only one available right now is TraversoCommands
+	commandname:	The Command to be created, which by convention tries to be the same name as the actuall class implementing the Command
+	arguments:	One, or more values (seperated by semicolons), each Command explains which (list of) values it can understand
+	\endcode 
+
+
+	Available Command classes, their required Object (to operate on) and arguments are:
+
+
+	\code 
+	MoveClip: Move or Copy-move an AudioClip.
+	
+	objectname:	AudioClipView
+	arguments:	First entry, either one of the following: "move", "copy", "move_to_end", "move_to_start"
+			Second (optional) entry: Move vertical only: "true" for vertical moving only, "false" for omnidirectional moving (the default if none is given)
+	commandname:	MoveClip
+	\endcode 
+
+
+	\code 
+	Gain: Change (jog), or reset the Gain of audio processing items
+	objectname:	SheetView, TrackView, AudioClipView
+	arguments:	No argument: Move Mouse vertically. Argument "horizontal": Move Mouse horizontally
+	commandname:	Gain
+	\endcode 
+	
+	\code 
+	ResetGain: Set Gain value to a pre-defined value (by means of the supplied argument).
+	
+	objectname:	SheetView, TrackView, AudioClipView
+	arguments:	a floating point value, eg: "0.5"
+	commandname:	ResetGain
+	\endcode 
+	
+	TODO document all remaining Commands
+	\code 
+	?: ?
+	
+	Objects(s):	
+	arguments:	
+	commandname:	
+	\endcode 
+
+ */
+
+
 TraversoCommands::TraversoCommands()
 {
 	m_dict.insert("Gain", GainCommand);
@@ -43,14 +108,30 @@ TraversoCommands::TraversoCommands()
 	m_dict.insert("RemoveTrack", RemoveTrackCommand);
 	m_dict.insert("AudioClipExternalProcessing", AudioClipExternalProcessingCommand);
 	m_dict.insert("ClipSelectionSelect", ClipSelectionCommand);
+	m_dict.insert("ClipSelectionSelectAll", ClipSelectionCommand);
 	m_dict.insert("ClipSelectionAdd", ClipSelectionCommand);
 	m_dict.insert("ClipSelectionRemove", ClipSelectionCommand);
 	m_dict.insert("MoveClip", MoveClipCommand);
+	m_dict.insert("FoldSheet", MoveClipCommand);
+	m_dict.insert("FoldTrack", MoveClipCommand);
+	m_dict.insert("FoldMarkers", MoveClipCommand);
 	m_dict.insert("DragEdge", DragEdgeCommand);
 	m_dict.insert("MoveClipOrEdge", MoveClipOrEdgeCommand);
 	m_dict.insert("CopyClip", MoveClipCommand);
-	m_dict.insert("SplitClip", SplitClipCommand);
-	m_dict.insert("ArmTracks", ArmTracksCommand);
+        m_dict.insert("SplitClip", SplitClipCommand);
+        m_dict.insert("CropClip", CropClipCommand);
+        m_dict.insert("ArmTracks", ArmTracksCommand);
+	m_dict.insert("VZoomIn", ZoomCommand);
+	m_dict.insert("VZoomOut", ZoomCommand);
+	m_dict.insert("HZoomIn", ZoomCommand);
+	m_dict.insert("HZoomOut", ZoomCommand);
+	m_dict.insert("HJogZoom", ZoomCommand);
+	m_dict.insert("VJogZoom", ZoomCommand);
+	m_dict.insert("JogZoom", ZoomCommand);
+	m_dict.insert("ScrollRightHold", ScrollCommand);
+	m_dict.insert("ScrollLeftHold", ScrollCommand);
+	m_dict.insert("ScrollUpHold", ScrollCommand);
+	m_dict.insert("ScrollDownHold", ScrollCommand);
 }
 
 Command* TraversoCommands::create(QObject* obj, const QString& command, QVariantList arguments)
@@ -59,27 +140,27 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 		case GainCommand:
 		{
 			ContextItem* item = qobject_cast<ContextItem*>(obj);
-			SongView* songview = 0;
+			SheetView* sheetview = 0;
 			
 			if (item->metaObject()->className() == QString("TrackPanelGain")) {
 				item = item->get_context();
-			} else if (item->metaObject()->className() == QString("SongPanelGain")) {
+			} else if (item->metaObject()->className() == QString("SheetPanelGain")) {
 				item = item->get_context();
 			} else if (AudioClipView* view = qobject_cast<AudioClipView*>(item)) {
-				songview = view->get_songview();
+				sheetview = view->get_sheetview();
 				item = view->get_context();
 			} else if (TrackView* view = qobject_cast<TrackView*>(item)) {
-				songview = view->get_songview();
+				sheetview = view->get_sheetview();
 				item = view->get_context();
-			} else if (SongView* view = qobject_cast<SongView*>(item)) {
-				songview = view;
+			} else if (SheetView* view = qobject_cast<SheetView*>(item)) {
+				sheetview = view;
 				item = view->get_context();
 			}
 			
-			// ugly hack to avoid assigning a songview when the 
+			// ugly hack to avoid assigning a sheetview when the 
 			// mouse cursor is above the trackpanel....
 			if (cpointer().scene_x() < 0) {
-				songview = 0;
+				sheetview = 0;
 			}
 			
 			if (!item) {
@@ -87,7 +168,7 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 					"GainCommand only works with ContextItem objects!!");
 				return 0;
 			}
-			return new Gain(item, songview, arguments);
+			return new Gain(item, sheetview, arguments);
 		}
 		
 		case TrackPanCommand:
@@ -130,13 +211,13 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 		
 		case AddNewTrackCommand:
 		{
-			Song* song = qobject_cast<Song*>(obj);
-			if (!song) {
-				PERROR("TraversoCommands: Supplied QObject was not a Song! "
-					"AddNewTrackCommand needs a Song as argument");
+			Sheet* sheet = qobject_cast<Sheet*>(obj);
+			if (!sheet) {
+				PERROR("TraversoCommands: Supplied QObject was not a Sheet! "
+					"AddNewTrackCommand needs a Sheet as argument");
 				return 0;
 			}
-			return song->add_track(new Track(song, "Unnamed", Track::INITIAL_HEIGHT));
+			return sheet->add_track(new Track(sheet, "Unnamed", Track::INITIAL_HEIGHT));
 		}
 		
 		case RemoveClipCommand:
@@ -158,7 +239,7 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 					"RemoveTrackCommand needs a Track as argument");
 				return 0;
 			}
-			return track->get_song()->remove_track(track);
+			return track->get_sheet()->remove_track(track);
 		}
 		
 		case AudioClipExternalProcessingCommand:
@@ -174,6 +255,16 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 		
 		case ClipSelectionCommand:
 		{
+			Sheet* sheet = qobject_cast<Sheet*>(obj);
+			if (sheet) {
+				QString action;
+				if (arguments.size()) {
+					action = arguments.at(0).toString();
+					if (action == "select_all_clips") {
+						return sheet->get_audioclip_manager()->select_all_clips();
+					}
+				}
+			}
 			AudioClip* clip = qobject_cast<AudioClip*>(obj);
 			if (!clip) {
 				PERROR("TraversoCommands: Supplied QObject was not an AudioClip! "
@@ -185,24 +276,14 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 		
 		case MoveClipCommand:
 		{
-			AudioClipView* view = qobject_cast<AudioClipView*>(obj);
+			ViewItem* view = qobject_cast<ViewItem*>(obj);
 			if (!view) {
 				PERROR("TraversoCommands: Supplied QObject was not an AudioClipView! "
 					"MoveClipCommand needs an AudioClipView as argument");
 				return 0;
 			}
 
-			if (view->get_clip()->is_locked()) {
-				return 0;
-			}
-
-			QString type;
-			if (arguments.size()) {
-				type = arguments.at(0).toString();
-			} else {
-				type = "move";  // Default Action
-			}
-			return new MoveClip(view, type);
+			return new MoveClip(view, arguments);
 		}
 		
 		case DragEdgeCommand:
@@ -213,25 +294,18 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 					"DragEdgeCommand needs an AudioClipView as argument");
 				return 0;
 			}
-
+			
 			int x = (int) (cpointer().on_first_input_event_scene_x() - view->scenePos().x());
-			bool anchorAudio = arguments[0].toBool();
-
+			
 			if (x < (view->boundingRect().width() / 2)) {
-				if (anchorAudio) {
-					return new MoveClip(view, "anchored_left_edge_move");
-				} else {
-					return new MoveEdge(view, view->get_songview(), "set_left_edge");
-				}
+				return new MoveEdge(view, view->get_sheetview(), "set_left_edge");
 			} else {
-				if (anchorAudio) {
-					return new MoveClip(view, "anchored_right_edge_move");
-				} else {
-					return new MoveEdge(view, view->get_songview(), "set_right_edge");
-				}
+				return new MoveEdge(view, view->get_sheetview(), "set_right_edge");
 			}
 		}
 
+		// The existence of this is doubtfull. Using [ E ] is so much easier
+		// then trying to mimic 'if near to edge, drag edge' features.
 		case MoveClipOrEdgeCommand:
 		{
 			AudioClipView* view = qobject_cast<AudioClipView*>(obj);
@@ -241,25 +315,21 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 					"MoveClipOrEdgeCommand needs an AudioClipView as argument");
 				return 0;
 			}
-
+			
 			int x = (int) (cpointer().on_first_input_event_scene_x() - view->scenePos().x());
-			int edge_width = arguments[0].toInt();
-			bool anchorAudio = arguments[1].toBool();
-
-			if (x < edge_width) {
-				if (anchorAudio) {
-					return new MoveClip(view, "anchored_left_edge_move");
-				} else {
-					return new MoveEdge(view, view->get_songview(), "set_left_edge");
-				}
-			} else if (x > (view->boundingRect().width() - edge_width)) {
-				if (anchorAudio) {
-					return new MoveClip(view, "anchored_right_edge_move");
-				} else {
-					return new MoveEdge(view, view->get_songview(), "set_right_edge");
-				}
+			
+			int edge_width = 0;
+			if (arguments.size() == 2) {
+				edge_width = arguments[0].toInt();
 			}
-			return new MoveClip(view, "move");
+			
+			if (x < edge_width) {
+				return new MoveEdge(view, view->get_sheetview(), "set_left_edge");
+			} else if (x > (view->boundingRect().width() - edge_width)) {
+				return new MoveEdge(view, view->get_sheetview(), "set_right_edge");
+			}
+			
+			return new MoveClip(view, QVariantList() << "move");
 		}
 
 		case SplitClipCommand:
@@ -273,16 +343,48 @@ Command* TraversoCommands::create(QObject* obj, const QString& command, QVariant
 			return new SplitClip(view);
 		}
 		
-		case ArmTracksCommand:
+                case CropClipCommand:
+                {
+                        AudioClipView* view = qobject_cast<AudioClipView*>(obj);
+                        if (!view) {
+                                PERROR("TraversoCommands: Supplied QObject was not an AudioClipView! "
+                                        "CropClipCommand needs an AudioClipView as argument");
+                                return 0;
+                        }
+                        return new Crop(view);
+                }
+
+                case ArmTracksCommand:
 		{
-			SongView* view = qobject_cast<SongView*>(obj);
+			SheetView* view = qobject_cast<SheetView*>(obj);
 			if (!view) {
-				PERROR("TraversoCommands: Supplied QObject was not an SongView! "
-						"ArmTracksCommand needs an SongView as argument");
+				PERROR("TraversoCommands: Supplied QObject was not an SheetView! "
+						"ArmTracksCommand needs an SheetView as argument");
 				return 0;
 			}
 			return new ArmTracks(view);
 		}
+		
+		case ZoomCommand:
+		{
+			SheetView* view = qobject_cast<SheetView*>(obj);
+			if (!view) {
+				PERROR("TraversoCommands: Supplied QObject was not an SheetView! "
+						"ArmTracksCommand needs an SheetView as argument");
+				return 0;
+			}
+			return new Zoom(view, arguments);
+		}
+		case ScrollCommand:
+		{
+			SheetView* view = qobject_cast<SheetView*>(obj);
+			if (!view) {
+				PERROR("TraversoCommands: Supplied QObject was not an SheetView! "
+						"ArmTracksCommand needs an SheetView as argument");
+				return 0;
+			}
+			return new Scroll(view, arguments);
+		}
 	}
 	
 	return 0;
