@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2007 Remon Sijrier 
+Copyright (C) 2007-2008 Remon Sijrier 
 
 This file is part of Traverso
 
@@ -39,15 +39,29 @@ class TraversoCommands : public CommandPlugin
 	Q_CLASSINFO("RemoveClip", tr("Remove Clip"))
 	Q_CLASSINFO("RemoveTrack", tr("Remove Track"))
 	Q_CLASSINFO("AudioClipExternalProcessing", tr("External Processing"))
-	Q_CLASSINFO("ClipSelectionSelect", tr("Select"))
-	Q_CLASSINFO("ClipSelectionRemove", tr("Remove from Selection"))
-	Q_CLASSINFO("ClipSelectionAdd", tr("Add to Selection"))
+	Q_CLASSINFO("ClipSelectionSelect", tr("(De)Select"))
+	Q_CLASSINFO("ClipSelectionSelectAll", tr("(De)Select All"))
 	Q_CLASSINFO("MoveClip", tr("Move Clip"))
 	Q_CLASSINFO("DragEdge", tr("Drag Edge"))
 	Q_CLASSINFO("MoveClipOrEdge", tr("Move Or Resize Clip"))
-	Q_CLASSINFO("SplitClip", tr("Split"))
-	Q_CLASSINFO("ArmTracks", tr("Arm Tracks"))
-
+        Q_CLASSINFO("SplitClip", tr("Split"))
+        Q_CLASSINFO("CropClip", tr("Magnetic Cut"))
+        Q_CLASSINFO("ArmTracks", tr("Arm Tracks"))
+	Q_CLASSINFO("FoldSheet", tr("Fold Sheet"))
+	Q_CLASSINFO("FoldTrack", tr("Fold Track"))
+	Q_CLASSINFO("FoldMarkers", tr("Fold Markers"))
+	Q_CLASSINFO("VZoomIn", tr("Vertical In"))
+	Q_CLASSINFO("HZoomOut", tr("Horizontal Out"))
+	Q_CLASSINFO("HZoomIn", tr("Horizontal In"))
+	Q_CLASSINFO("VZoomOut", tr("Vertical Out"))
+	Q_CLASSINFO("Zoom", tr("Omnidirectional"))
+	Q_CLASSINFO("HJogZoom", tr("Horizontal"))
+	Q_CLASSINFO("VJogZoom", tr("Vertical"))
+	Q_CLASSINFO("ScrollRightHold", tr("Right"))
+	Q_CLASSINFO("ScrollLeftHold", tr("Left"))
+	Q_CLASSINFO("ScrollUpHold", tr("Up"))
+	Q_CLASSINFO("ScrollDownHold", tr("Down"))
+	
 public:
 	TraversoCommands();
 	Command* create(QObject* obj, const QString& command, QVariantList arguments);
@@ -67,7 +81,10 @@ private:
     		DragEdgeCommand,
     		MoveClipOrEdgeCommand,
       		SplitClipCommand,
-		ArmTracksCommand
+                CropClipCommand,
+		ArmTracksCommand,
+  		ZoomCommand,
+    		ScrollCommand
 	};
 };
 
