@@ -510,7 +510,7 @@ int AudioClip::process(nframes_t nframes)
 	
 
 	apill_foreach(FadeCurve* fade, FadeCurve, m_fades) {
-		fade->process(mixdown, read_frames, channelcount);
+		fade->process(bus, nframes);
 	}
 	
 	TimeRef endlocation = mix_pos + TimeRef(read_frames, get_rate());
