@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2006 Remon Sijrier 
+Copyright (C) 2005-2008 Remon Sijrier 
 
 This file is part of Traverso
 
@@ -28,13 +28,12 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
 #include "ContextItem.h"
 #include "AudioProcessingItem.h"
-#include "SnapList.h"
 #include "Snappable.h"
 #include "defines.h"
 #include "GainEnvelope.h"
 
 
-class Song;
+class Sheet;
 class ReadSource;
 class WriteSource;
 class Track;
@@ -47,11 +46,10 @@ class AudioClip : public ContextItem, public AudioProcessingItem, public Snappab
 {
 	Q_OBJECT
 	Q_CLASSINFO("mute", tr("Mute"))
-	Q_CLASSINFO("reset_fade_in", tr("In: Reset"))
-	Q_CLASSINFO("reset_fade_out", tr("Out: Reset"))
-	Q_CLASSINFO("reset_fade_both", tr("Reset both"))
+	Q_CLASSINFO("reset_fade_in", tr("In: Remove"))
+	Q_CLASSINFO("reset_fade_out", tr("Out: Remove"))
+	Q_CLASSINFO("reset_fade_both", tr("Both: Remove"))
 	Q_CLASSINFO("normalize", tr("Normalize"))
-	Q_CLASSINFO("denormalize", tr("Normalize: reset"))
 	Q_CLASSINFO("lock", tr("Lock"))
 
 public:
@@ -75,49 +73,51 @@ public:
 	void set_fade_in(double range);
 	void set_fade_out(double range);
 	void set_track(Track* track);
-	void set_song(Song* song);
+	void set_sheet(Sheet* sheet);
 
 	void set_selected(bool selected);
+	void set_as_moving(bool moving);
 	int set_state( const QDomNode& node );
 
 	AudioClip* create_copy();
 	Track* get_track() const;
-	Song* get_song() const;
+	Sheet* get_sheet() const;
 	Peak* get_peak() const {return m_peak;}
 	QDomNode get_state(QDomDocument doc);
 	FadeCurve* get_fade_in() const;
 	FadeCurve* get_fade_out() const;
 	PluginChain* get_plugin_chain() const {return m_pluginChain;}
 	
-	TimeRef& get_length() const;
-	TimeRef& get_source_length() const;
-	TimeRef& get_track_start_location() const {return m_trackStartLocation;}
-	TimeRef& get_track_end_location() const {return m_trackEndLocation;}
-	TimeRef& get_source_start_location() const {return m_sourceStartLocation;}
-	TimeRef& get_source_end_location() const {return m_sourceEndLocation;}
+	TimeRef get_source_length() const;
+	TimeRef get_length() const {return m_length;}
+	TimeRef get_track_start_location() const {return m_trackStartLocation;}
+	TimeRef get_track_end_location() const {return m_trackEndLocation;}
+	TimeRef get_source_start_location() const {return m_sourceStartLocation;}
+	TimeRef get_source_end_location() const {return m_sourceEndLocation;}
 	
 	int get_channels() const;
 	int get_rate() const;
 	int get_bitdepth() const;
 	qint64 get_readsource_id() const;
-	qint64 get_song_id() const {return m_songId;}
+	qint64 get_sheet_id() const {return m_sheetId;}
 	ReadSource* get_readsource() const;
 	
 	QString get_name() const;
 	QDomNode get_dom_node() const;
 	
 	bool is_take() const;
-	bool is_selected() const;
+	bool is_selected();
 	bool is_locked() const {return m_isLocked;}
-	bool has_song() const;
+	bool has_sheet() const;
 	bool is_readsource_invalid() const {return !m_isReadSourceValid;}
 	bool is_smaller_then(APILinkedListNode* node) {return ((AudioClip*)node)->get_track_start_location() > get_track_start_location();}
+	bool is_moving() const {return m_isMoving;}
 
 	int recording_state() const;
 
 private:
 	Track* 			m_track;
-	Song* 			m_song;
+	Sheet* 			m_sheet;
 	ReadSource*		m_readSource;
 	WriteSource*		m_recorder;
 	APILinkedList		m_fades;
@@ -130,21 +130,21 @@ private:
 	QString 		m_name;
 	QByteArray		m_captureBusName;
 	
-	mutable TimeRef 	m_trackStartLocation;
-	mutable TimeRef 	m_trackEndLocation;
-	mutable TimeRef 	m_sourceEndLocation;
-	mutable TimeRef 	m_sourceStartLocation;
-	mutable TimeRef		m_sourceLength;
-	mutable TimeRef 	m_length;
+	TimeRef 		m_trackStartLocation;
+	TimeRef 		m_trackEndLocation;
+	TimeRef 		m_sourceEndLocation;
+	TimeRef 		m_sourceStartLocation;
+	TimeRef			m_sourceLength;
+	TimeRef 		m_length;
 
-	int 			m_isSelected;
 	bool 			m_isTake;
 	bool			m_isLocked;
 	bool			m_isReadSourceValid;
+	bool			m_isMoving;
 	RecordingStatus		m_recordingStatus;
 	
 	qint64			m_readSourceId;
-	qint64			m_songId;
+	qint64			m_sheetId;
 
 	void create_fade_in();
 	void create_fade_out();
@@ -156,7 +156,7 @@ private:
 	void process_capture(nframes_t nframes);
 	
 	
-	void calculate_normalization_factor(float targetdB = 0.0);
+        float calculate_normalization_factor(float targetdB = 0.0);
 	
 	friend class ResourcesManager;
 
@@ -164,8 +164,7 @@ signals:
 	void stateChanged();
 	void muteChanged();
 	void lockChanged();
-	void positionChanged(Snappable*);
-	void trackEndLocationChanged();
+	void positionChanged();
 	void fadeAdded(FadeCurve*);
 	void fadeRemoved(FadeCurve*);
 	void recordingFinished(AudioClip*);
@@ -187,7 +186,6 @@ public slots:
 	Command* reset_fade_out();
 	Command* reset_fade_both();
         Command* normalize();
-        Command* denormalize();
 	Command* lock();
 
 private slots:
