@@ -17,20 +17,21 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: FadeCurve.cpp,v 1.31 2007/11/19 11:18:53 r_sijrier Exp $
+$Id: FadeCurve.cpp,v 1.36 2008/11/07 10:43:08 r_sijrier Exp $
 */
  
 #include "FadeCurve.h"
 
 #include <QFile>
 #include <cmath>
-#include "Song.h"
+#include "Sheet.h"
 #include "Fade.h"
 #include "AudioClip.h"
 #include "Command.h"
 #include "CommandGroup.h"
 #include <AddRemove.h>
 #include "AudioDevice.h"
+#include "AudioBus.h"
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
@@ -39,12 +40,12 @@ $Id: FadeCurve.cpp,v 1.31 2007/11/19 11:18:53 r_sijrier Exp $
 
 QStringList FadeCurve::defaultShapes = QStringList() << "Fastest" << "Fast" << "Linear"  << "Slow" << "Slowest";
 
-FadeCurve::FadeCurve(AudioClip* clip, Song* song, QString type )
+FadeCurve::FadeCurve(AudioClip* clip, Sheet* sheet, QString type )
 	: Curve(clip)
 	, m_clip(clip)
 	, m_sType(type)
 {
-	Q_UNUSED(song);
+	Q_UNUSED(sheet);
 	
 	if (type == "FadeIn") {
 		m_type = FadeIn;
@@ -53,25 +54,25 @@ FadeCurve::FadeCurve(AudioClip* clip, Song* song, QString type )
 		m_type = FadeOut;
 	}
 	
-	m_song = m_clip->get_song();
+	m_sheet = m_clip->get_sheet();
 		
 	m_controlPoints.append(QPointF(0.0, 0.0));
 	m_controlPoints.append(QPointF(0.25, 0.25));
 	m_controlPoints.append(QPointF(0.75, 0.75));
 	m_controlPoints.append(QPointF(1.0, 1.0));
 	
-	m_bendFactor = 0.5;
-	m_strenghtFactor = 1;
+	m_bendFactor = 0.0;
+	m_strenghtFactor = 0.5;
 	m_mode = 2;
 	m_raster = 0;
 	m_bypass = false;
 	
-	init();
-	
 	connect(this, SIGNAL(stateChanged()), this, SLOT(solve_node_positions()));
 	connect(this, SIGNAL(bendValueChanged()), this, SIGNAL(stateChanged()));
 	connect(this, SIGNAL(strengthValueChanged()), this, SIGNAL(stateChanged()));
 	connect(this, SIGNAL(modeChanged()), this, SIGNAL(stateChanged()));
+
+	init();
 }
 
 FadeCurve::~ FadeCurve( )
@@ -81,6 +82,8 @@ FadeCurve::~ FadeCurve( )
 
 void FadeCurve::init()
 {
+	QObject::tr("FadeCurve");
+	
 	// Populate the curve with 12 CurveNodes
 	float f = 0.0;
 	int nodecount = 11;
@@ -126,10 +129,10 @@ QDomNode FadeCurve::get_state( QDomDocument doc )
 int FadeCurve::set_state( const QDomNode & node )
 {
 	QDomElement e = node.toElement();
-	m_bendFactor = e.attribute( "bendfactor", "0.5" ).toDouble();
+	m_bendFactor = e.attribute( "bendfactor", "0.0" ).toDouble();
 	m_strenghtFactor = e.attribute( "strengthfactor", "0.5" ).toDouble();
 	m_bypass = e.attribute( "bypassed", "0" ).toInt();
-	m_mode = e.attribute( "mode", "0" ).toInt();
+	m_mode = e.attribute( "mode", "2" ).toInt();
 	m_raster = e.attribute( "raster", "0" ).toInt();
 	
 	QStringList controlPointsList = e.attribute( "controlpoints", "0.0,0.0;0.25,0.25;0.75,0.75;1.0,1.0" ).split(";");
@@ -159,48 +162,81 @@ int FadeCurve::set_state( const QDomNode & node )
 	solve_node_positions();	
 	
 	emit stateChanged();
-	
+
 	return 1;
 }
 
 
-void FadeCurve::process(audio_sample_t** mixdown, nframes_t nframes, uint channels)
+void FadeCurve::process(AudioBus *bus, nframes_t nframes)
 {
 
-	if (is_bypassed() || (get_range() < 16)) {
+        if (is_bypassed()) {
 		return;
 	}
 	
 	
-	TimeRef faderange = TimeRef(get_range());
-	TimeRef fadepos;
+        audio_sample_t* mixdown[bus->get_channel_count()];
+        int outputRate = audiodevice().get_sample_rate();
+        uint framesToProcess = nframes;
+
+        TimeRef trackStartLocation, trackEndLocation, mix_pos;
+        TimeRef fadeRange = TimeRef(get_range());
+
+        TimeRef transportLocation = m_sheet->get_transport_location();
+        TimeRef upperRange = transportLocation + TimeRef(framesToProcess, outputRate);
+
 	
 	if (m_type == FadeIn) {
-		if( !( m_song->get_transport_location() < (m_clip->get_track_start_location() + faderange) ) ) {
-			return;
-		}
-		
-		fadepos = m_song->get_transport_location() - m_clip->get_track_start_location();
-	} else {
-		if( !(m_song->get_transport_location() > (m_clip->get_track_end_location() - faderange)) ) {
-			return;
-		}
-		
-		fadepos = m_song->get_transport_location() - (m_clip->get_track_end_location() - faderange);
+                trackStartLocation = m_clip->get_track_start_location();
+        } else {
+                trackStartLocation = m_clip->get_track_end_location() - fadeRange;
 	}
 
-	TimeRef range(nframes, audiodevice().get_sample_rate());
-	TimeRef limit(std::min (range.universal_frame(), faderange.universal_frame()));
-	
-	nframes_t framerange = limit.to_frame(audiodevice().get_sample_rate());
-	
-	get_vector(fadepos.universal_frame(), (fadepos + limit).universal_frame(), m_song->gainbuffer, framerange);
+        trackEndLocation = trackStartLocation + fadeRange;
 
-	for (uint chan=0; chan<channels; ++chan) {
-		for (nframes_t frame = 0; frame < framerange; ++frame) {
-			mixdown[chan][frame] *= m_song->gainbuffer[frame];
-		}
-	}
+
+        if ( (trackStartLocation < upperRange) && (trackEndLocation > transportLocation) ) {
+                if (transportLocation < trackStartLocation) {
+                        // Using to_frame() for both the m_trackStartLocation and transportLocation seems to round
+                        // better then using (m_trackStartLocation - transportLocation).to_frame()
+                        // TODO : find out why!
+                        uint offset = (trackStartLocation).to_frame(outputRate) - transportLocation.to_frame(outputRate);
+                        mix_pos = TimeRef();
+//                        printf("offset %d\n", offset);
+
+                        for (int chan=0; chan<bus->get_channel_count(); ++chan) {
+                                audio_sample_t* buf = bus->get_buffer(chan, framesToProcess);
+                                mixdown[chan] = buf + offset;
+                        }
+                        framesToProcess = framesToProcess - offset;
+                } else {
+                        mix_pos = (transportLocation - trackStartLocation);
+
+                        for (int chan=0; chan<bus->get_channel_count(); ++chan) {
+                                mixdown[chan] = bus->get_buffer(chan, framesToProcess);
+                        }
+                }
+                if (trackEndLocation < upperRange) {
+                        // Using to_frame() for both the upperRange and m_trackEndLocation seems to round
+                        // better then using (upperRange - m_trackEndLocation).to_frame()
+                        // TODO : find out why!
+                        framesToProcess -= upperRange.to_frame(outputRate) - trackEndLocation.to_frame(outputRate);
+// 			printf("if (m_trackEndLocation < upperRange): framesToProcess %d\n", framesToProcess);
+                }
+        } else {
+                return;
+        }
+
+
+        upperRange = mix_pos + TimeRef(framesToProcess, outputRate);
+
+        get_vector(mix_pos.universal_frame(), upperRange.universal_frame(), m_sheet->gainbuffer, framesToProcess);
+
+        for (int chan=0; chan<bus->get_channel_count(); ++chan) {
+                for (nframes_t frame = 0; frame < framesToProcess; ++frame) {
+                        mixdown[chan][frame] *= m_sheet->gainbuffer[frame];
+                }
+        }
 }
 
 
@@ -236,22 +272,13 @@ void FadeCurve::set_shape(QString shapeName)
 		return;
 	}
 	
-	CommandGroup* group = new CommandGroup(this, tr("Fade Shape"));
+	CommandGroup* group = new CommandGroup(this, tr("Fade Preset"));
 	
-	group->add_command(new FadeBend(this, fadeElement.attribute( "bendfactor", "0.5" ).toDouble()));
+	group->add_command(new FadeBend(this, fadeElement.attribute( "bendfactor", "0.0" ).toDouble()));
 	group->add_command(new FadeStrength(this, fadeElement.attribute( "strengthfactor", "0.5" ).toDouble()));
 	
 	Command::process_command(group);
 	
-	QStringList controlPointsList = fadeElement.attribute( "controlpoints", "" ).split(";");
-	
-	for (int i=0; i<controlPointsList.size(); ++i) {
-		QStringList xyList = controlPointsList.at(i).split(",");
-		float x = xyList.at(0).toFloat();
-		float y = xyList.at(1).toFloat();
-		m_controlPoints[i] = QPointF(x,y);
-	}
-	
 	emit stateChanged();
 }
 
@@ -375,10 +402,7 @@ void FadeCurve::set_mode(int m)
 
 Command* FadeCurve::reset( )
 {
-	set_bend_factor(0.5);
-	set_strength_factor(0.5);
-	
-	return 0;
+	return (m_type == FadeIn) ? m_clip->reset_fade_in() : m_clip->reset_fade_out();
 }
 
 Command* FadeCurve::toggle_bypass( )
