@@ -17,19 +17,21 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: FileHelpers.h,v 1.2 2006/05/17 22:01:10 r_sijrier Exp $
+$Id: FileHelpers.h,v 1.4 2007/10/06 14:17:58 r_sijrier Exp $
 */
 
+#ifndef FILE_HELPER_H
+#define FILE_HELPER_H
+
 #include <QString>
 
 class FileHelper
 {
 public:
 
-	static int remove_recursively(QString pName);
-
-	static int copy_recursively(QString pNameFrom, QString pNameTo);
-
+	static int remove_recursively(const QString& pName);
+	static int copy_recursively(const QString& pNameFrom, const QString& pNameTo);
+	static QString fileerror_to_string(int error);
 };
 
-//eof
+#endif
