@@ -28,18 +28,13 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "ResourcesManager.h"
 #include "defines.h"
 #include "Mixer.h"
+#include "FileHelpers.h"
 #include <QFileInfo>
 #include <QDateTime>
 #include <QMutexLocker>
 
 #include "Debugger.h"
 
-/* Store for each zoomStep the upper and lower sample to hard disk as a
-* peak_data_t. The top-top resolution is then 512 pixels, which should do
-* Painting the waveform will be as simple as painting a line starting from the
-* lower value to the upper value.
-*/
-
 #define NORMALIZE_CHUNK_SIZE	10000
 #define PEAKFILE_MAJOR_VERSION	1
 #define PEAKFILE_MINOR_VERSION	4
@@ -51,6 +46,8 @@ int Peak::zoomStep[] = {
  	64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576 
 };
 
+QHash<int, int> Peak::chacheIndexLut;
+
 Peak::Peak(AudioSource* source)
 {
 	PENTERCONS;
@@ -82,7 +79,6 @@ Peak::Peak(AudioSource* source)
 	if (rs) {
 		// This Peak object was created by AudioClip, meant for reading peak data
 		m_source = resources_manager()->get_readsource(rs->get_id());
-		m_source->set_output_rate(44100, true);
 	} else {
 		// No ReadSource object? Then it's created by WriteSource for on the fly
 		// peak data creation, no m_source needed!
@@ -130,10 +126,15 @@ int Peak::read_header()
 	
 	foreach(ChannelData* data, m_channelData) {
 		
-		data->file.setFileName(data->fileName.toUtf8().data());
+		data->file.setFileName(data->fileName);
 		
 		if (! data->file.open(QIODevice::ReadOnly)) {
-			PERROR("Couldn't open peak file for reading! (%s)", data->fileName.toAscii().data());
+			if (QFile::exists(data->fileName)) {
+				m_permanentFailure = true;
+			}
+				
+			QString errorstring = FileHelper::fileerror_to_string(data->file.error());
+			qWarning("Couldn't open peak file for reading! (%s, Error: %s)", QS_C(data->fileName), QS_C(errorstring));
 			return -1;
 		}
 
@@ -236,32 +237,12 @@ void Peak::start_peak_loading()
 }
 
 
-// static int
-// cnt_bits(unsigned long val, int & highbit)
-// {
-// 	int cnt = 0;
-// 	highbit = 0;
-// 	while (val) {
-// 		if (val & 1) cnt++;
-// 		val>>=1;
-// 		highbit++;
-// 	}
-// 	return cnt;
-// }
-// 
-// // returns the next power of two greater or equal to val
-// static unsigned long
-// nearest_power_of_two(unsigned long val)
-// {
-// 	int highbit;
-// 	if (cnt_bits(val, highbit) > 1) {
-// 		return 1<<highbit;
-// 	}
-// 	return val;
-// }
-
-
-int Peak::calculate_peaks(int chan, float** buffer, int zoomLevel, TimeRef startlocation, int pixelcount)
+int Peak::calculate_peaks(
+	int chan,
+	float ** buffer,
+	TimeRef startlocation,
+	int peakDataCount,
+	qreal framesPerPeak)
 {
 	PENTER3;
 	
@@ -275,52 +256,65 @@ int Peak::calculate_peaks(int chan, float** buffer, int zoomLevel, TimeRef start
 		}
 	}
 	
-	if (pixelcount <= 0) {
+	if (peakDataCount <= 0) {
 		return NO_PEAKDATA_FOUND;
 	}
 	
 	ChannelData* data = m_channelData.at(chan);
+	int produced = 0;
 	
 // 	PROFILE_START;
 	
 	// Macro view mode
-	if ( zoomLevel > MAX_ZOOM_USING_SOURCEFILE) {
+	if (framesPerPeak >= 64) {
+		int highbit;
+		unsigned long nearestpow2 = nearest_power_of_two(qRound(framesPerPeak), highbit);
+		if (nearestpow2 == 0) {
+			return NO_PEAKDATA_FOUND;
+		}
+		
+		
 		nframes_t startPos = startlocation.to_frame(44100);
 		
-		int offset = (startPos / zoomStep[zoomLevel]) * 2;
+		int index = cache_index_lut()->value(nearestpow2, -1);
+		if(index >= 0) {
+// 			printf("index %d\n", index);
+		}
+		
+		int offset = qRound(startPos / nearestpow2) * 2;
 		int truncate = 0;
 		
 		// Check if this zoom level has as many data as requested.
-		if ( (pixelcount + offset) > data->headerdata.peakDataSizeForLevel[zoomLevel - SAVING_ZOOM_FACTOR]) {
-			truncate = pixelcount - (data->headerdata.peakDataSizeForLevel[zoomLevel - SAVING_ZOOM_FACTOR] - offset);
-// 			pixelcount = data->headerdata.peakDataSizeForLevel[zoomLevel - SAVING_ZOOM_FACTOR] - offset;
+		if ( (peakDataCount + offset) > data->headerdata.peakDataSizeForLevel[index]) {
+			truncate = peakDataCount - (data->headerdata.peakDataSizeForLevel[index] - offset);
+// 			pixelcount = data->headerdata.peakDataSizeForLevel[index] - offset;
 		}
 		
-		nframes_t readposition = data->headerdata.headerSize + (data->headerdata.peakDataOffsets[zoomLevel - SAVING_ZOOM_FACTOR] + offset) * sizeof(peak_data_t);
-		int read = data->peakreader->read_from(data->peakdataDecodeBuffer, readposition, pixelcount);
+		nframes_t readposition = data->headerdata.headerSize + (data->headerdata.peakDataOffsets[index] + offset) * sizeof(peak_data_t);
+		produced = data->peakreader->read_from(data->peakdataDecodeBuffer, readposition, peakDataCount);
 		
-		if (read != pixelcount) {
-			PERROR("Could not read in all peak data, pixelcount is %d, read count is %d", pixelcount, read);
+		if (produced != peakDataCount) {
+			PERROR("Could not read in all peak data, peakDataCount is %d, read count is %d", peakDataCount, produced);
 		}
 		
 // 		PROFILE_END("Peak calculate_peaks");
 		
-		if (read == 0) {
+		if (produced == 0) {
 			return NO_PEAKDATA_FOUND;
 		}
 		
-		for (int i=(pixelcount-truncate); i<(pixelcount); ++i) {
-			data->peakdataDecodeBuffer->destination[0][i] = 0;
-		}
-
+// 		for (int i=(pixelcount-truncate); i<(pixelcount); ++i) {
+// 			data->peakdataDecodeBuffer->destination[0][i] = 0;
+// 		}
+// 		
 		*buffer = data->peakdataDecodeBuffer->destination[0];
 		
-		return read;
-		
+		return produced;
+
 	// Micro view mode
 	} else {
 		// Calculate the amount of frames to be read
-		nframes_t toRead = pixelcount * zoomStep[zoomLevel];
+		nframes_t toRead = qRound(peakDataCount * framesPerPeak);
 		
 		nframes_t readFrames = m_source->file_read(data->peakdataDecodeBuffer, startlocation, toRead);
 
@@ -330,42 +324,57 @@ int Peak::calculate_peaks(int chan, float** buffer, int zoomLevel, TimeRef start
 		
 		if ( readFrames != toRead) {
 			PWARN("Unable to read nframes %d (only %d available)", toRead, readFrames);
-			pixelcount = readFrames / zoomStep[zoomLevel];
 		}
 
 		int count = 0;
-		nframes_t pos = 0;
-		audio_sample_t valueMax, valueMin, sample;
+		audio_sample_t sample;
 		
 		// MicroView needs a buffer to store the calculated peakdata
 		// our decodebuffer's readbuffer is large enough for this purpose
 		// and it's no problem to use it at this point in the process chain.
 		float* peakdata = data->peakdataDecodeBuffer->readBuffer;
 
-		do {
-			valueMax = valueMin = 0;
+		ProcessData pd;
+		// the stepSize depends on the real file sample rate, Peak assumes 44100 Hz
+		// so if the file sample rate differs, the stepSize becomes the ratio of 
+		// the file sample rate and 44100
+		pd.stepSize = TimeRef(qreal(44100) / m_source->get_file_rate(), m_source->get_file_rate());
+		pd.processRange = TimeRef(framesPerPeak, m_source->get_file_rate());
 
-			for(int i=0; i < zoomStep[zoomLevel]; i++) {
-				Q_ASSERT(pos <= readFrames);
-				sample = data->peakdataDecodeBuffer->destination[chan][pos];
-				if (sample > valueMax)
-					valueMax = sample;
-				if (sample < valueMin)
-					valueMin = sample;
-				pos++;
+		for (uint i=0; i < readFrames; i++) {
+		
+			pd.processLocation += pd.stepSize;
+		
+			sample = data->peakdataDecodeBuffer->destination[chan][i];
+		
+			pd.normValue = f_max(pd.normValue, fabsf(sample));
+		
+			if (sample > pd.peakUpperValue) {
+				pd.peakUpperValue = sample;
 			}
-
-			if (valueMax > fabs(valueMin)) {
-				peakdata[count] = valueMax;
-			} else {
-				peakdata[count] = valueMin;
+		
+			if (sample < pd.peakLowerValue) {
+				pd.peakLowerValue = sample;
 			}
+		
+			if (pd.processLocation >= pd.nextDataPointLocation) {
+		
+				if (pd.peakUpperValue > fabs(pd.peakLowerValue)) {
+					peakdata[count] = pd.peakUpperValue;
+				} else {
+					peakdata[count] = pd.peakLowerValue;
+				}
+				
+				pd.peakUpperValue = -10.0;
+				pd.peakLowerValue = 10.0;
 			
-			count++;
+				pd.nextDataPointLocation += pd.processRange;
+				count++;
+			}
+		}
+		
+// 		printf("framesPerPeak, peakDataCount, generated, readFrames %f, %d, %d, %d\n", framesPerPeak, peakDataCount, count, readFrames);
 		
-		} while(count < pixelcount);
-
-
 // 		PROFILE_END("Peak calculate_peaks");		
 		
 		// Assign the supplied buffer to the 'real' peakdata buffer.
@@ -374,8 +383,7 @@ int Peak::calculate_peaks(int chan, float** buffer, int zoomLevel, TimeRef start
 		return count;
 	}
 
-	
-	return 1;
+	return 0;
 }
 
 
@@ -419,7 +427,8 @@ int Peak::prepare_processing(int rate)
 		data->file.seek(data->headerdata.headerSize);
 		
 		data->pd = new Peak::ProcessData;
-		data->pd->stepSize = TimeRef(1, rate);
+		data->pd->stepSize = TimeRef(nframes_t(1), rate);
+		data->pd->processRange = TimeRef(nframes_t(64), 44100);
 	}
 	
 	
@@ -626,8 +635,6 @@ int Peak::create_from_scratch()
 		}
 	}
 
-	m_source->set_output_rate(m_source->get_file_rate(), true);
-	
 	DecodeBuffer decodebuffer;
 	
 	do {
@@ -656,7 +663,6 @@ int Peak::create_from_scratch()
 			emit progress(progression);
 			data->pd->progress = progression;
 		}
-		
 	} while (totalReadFrames < m_source->get_nframes());
 
 
@@ -671,8 +677,6 @@ out:
 	 
 // 	PROFILE_END("Peak create from scratch");
 	
-	m_source->set_output_rate(44100, true);
-
 	return ret;
 }
 
@@ -686,7 +690,6 @@ audio_sample_t Peak::get_max_amplitude(TimeRef startlocation, TimeRef endlocatio
 		}
 	}
 	int rate = m_source->get_file_rate();
-	m_source->set_output_rate(rate, true);
 	nframes_t startframe = startlocation.to_frame(rate);
 	nframes_t endframe = endlocation.to_frame(rate);
 	int startpos = startframe / NORMALIZE_CHUNK_SIZE;
@@ -734,7 +737,7 @@ audio_sample_t Peak::get_max_amplitude(TimeRef startlocation, TimeRef endlocatio
 		int read = data->file.read((char*)readbuffer, sizeof(audio_sample_t) * count) / sizeof(audio_sample_t);
 	
 		if (read != (int)count) {
-			printf("could only read %d, %d requested\n", read, count);
+                        printf("Peak::get_max_amplitude: could only read %d, %d requested\n", read, count);
 		}
 	
 		maxamp = Mixer::compute_peak(readbuffer, read, maxamp);
@@ -742,8 +745,6 @@ audio_sample_t Peak::get_max_amplitude(TimeRef startlocation, TimeRef endlocatio
 	
 	delete [] readbuffer;
 	
-	m_source->set_output_rate(44100, true);
-	
 	return maxamp;
 }
 
@@ -877,6 +878,7 @@ PeakDataReader::PeakDataReader(Peak::ChannelData* data)
 {
 	m_d = data;
 	m_nframes = m_d->file.size();
+	m_readPos = 0;
 }
 
 
@@ -905,7 +907,7 @@ bool PeakDataReader::seek(nframes_t start)
 		// only seek if we didn't mmap 
 		if (!m_d->memory) {
 			if (!m_d->file.seek(start)) {
-				PERROR("PeakDataReader: could not seek to data point %d within %s", start, m_d->fileName.toUtf8().data());
+				qWarning("PeakDataReader: could not seek to data point %d within %s", start, QS_C(m_d->fileName));
 				return false;
 			}
 		}
@@ -924,7 +926,7 @@ nframes_t PeakDataReader::read(DecodeBuffer* buffer, nframes_t count)
 	}
 		
 	// Make sure the read buffer is big enough for this read
-	buffer->check_buffers_capacity(count*2, 1);
+	buffer->check_buffers_capacity(count*3, 1);
 	
 	Q_ASSERT(m_d->file.isOpen());
 	
@@ -952,3 +954,33 @@ nframes_t PeakDataReader::read(DecodeBuffer* buffer, nframes_t count)
 	return framesRead;
 }
 
+void Peak::calculate_lut_data()
+{
+	chacheIndexLut.insert(64     , 0);
+	chacheIndexLut.insert(128    , 1);
+	chacheIndexLut.insert(256    , 2);
+	chacheIndexLut.insert(512    , 3);
+	chacheIndexLut.insert(1024   , 4);
+	chacheIndexLut.insert(2048   , 5);
+	chacheIndexLut.insert(4096   , 6);
+	chacheIndexLut.insert(8192   , 7);
+	chacheIndexLut.insert(16384  , 8);
+	chacheIndexLut.insert(32768  , 9);
+	chacheIndexLut.insert(65536  , 10);
+	chacheIndexLut.insert(131072 , 11);
+	chacheIndexLut.insert(262144 , 12);
+	chacheIndexLut.insert(524288 , 13);
+	chacheIndexLut.insert(1048576, 14);
+}
+
+int Peak::max_zoom_value()
+{
+	return 1048576;
+}
+
+Peak::ChannelData::~ ChannelData()
+{
+	if (peakdataDecodeBuffer) {
+		delete peakdataDecodeBuffer;
+	}
+}
