@@ -31,7 +31,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
 
 #include "Project.h"
-#include "Song.h"
+#include "Sheet.h"
 #include "ContextPointer.h"
 #include "ResourcesManager.h"
 #include "Information.h"
@@ -62,7 +62,7 @@ ProjectManager::ProjectManager()
 	
 	m_watcher = new QFileSystemWatcher(0);
 
-	QString path = config().get_property("Project", "directory", getenv("HOME")).toString();
+	QString path = config().get_property("Project", "directory", "").toString();
 	set_current_project_dir(path);
 	
 	cpointer().add_contextitem(this);
@@ -140,7 +140,7 @@ void ProjectManager::set_current_project(Project* project)
 
 }
 
-Project* ProjectManager::create_new_project(int numSongs, int numTracks, const QString& projectName)
+Project* ProjectManager::create_new_project(int numSheets, int numTracks, const QString& projectName)
 {
 	PENTER;
 
@@ -154,7 +154,7 @@ Project* ProjectManager::create_new_project(int numSongs, int numTracks, const Q
 	
 	Project* newProject = new Project(projectName);
 
-	if (newProject->create(numSongs, numTracks) < 0) {
+	if (newProject->create(numSheets, numTracks) < 0) {
 		delete newProject;
 		info().critical(tr("Unable to create new Project %1").arg(projectName));
 		return 0;
@@ -296,18 +296,24 @@ Project * ProjectManager::get_project( )
 
 void ProjectManager::start()
 {
-	QString defaultpath = config().get_property("Project", "DefaultDirectory", "").toString();
-	QString projects_path = config().get_property("Project", "directory", defaultpath).toString();
+	QString projectsPath = config().get_property("Project", "directory", "/unknown/directory/").toString();
 
 	QDir dir;
-	if ( (projects_path.isEmpty()) || (!dir.exists(projects_path)) ) {
-		if (projects_path.isEmpty()) {
-			projects_path = QDir::homePath();
+	if ( (projectsPath.isEmpty()) || (!dir.exists(projectsPath)) ) {
+		if (projectsPath.isEmpty()) {
+			projectsPath = QDir::homePath();
 		}
 
 		QString newPath = QFileDialog::getExistingDirectory(0,
-				tr("Choose an existing or create a new Project Directory"),
-				   projects_path );
+				tr("Choose a directory to store your Projects in"),
+				   projectsPath );
+		
+		if (newPath.isEmpty()) {
+			QMessageBox::warning( 0, tr("Traverso - Warning"), 
+					      tr("No directory was selected, to retry open the 'Open Project Dialog' and "
+						 "click 'Select Project Directory' button\n"));
+			return;
+		} 
 		
 		QFileInfo fi(newPath);
 		if (dir.exists(newPath) && !fi.isWritable()) {
@@ -384,7 +390,7 @@ Command* ProjectManager::exit()
 	PENTER;
 	
 	if (currentProject) {
-		if (currentProject->get_songs().size() == 0) {
+		if (currentProject->get_sheets().size() == 0) {
 			// No sheets to unregister from the audiodevice,
 			// just save and quit:
 			set_current_project(0);
@@ -404,20 +410,20 @@ Command* ProjectManager::exit()
 	return (Command*) 0;
 }
 
-void ProjectManager::scheduled_for_deletion( Song * song )
+void ProjectManager::scheduled_for_deletion( Sheet * sheet )
 {
 	PENTER;
-	m_deletionSongList.append(song);
+	m_deletionSheetList.append(sheet);
 }
 
-void ProjectManager::delete_song( Song * song )
+void ProjectManager::delete_sheet( Sheet * sheet )
 {
 	PENTER;
-	m_deletionSongList.removeAll(song);
-	emit aboutToDelete(song);
-	delete song;
+	m_deletionSheetList.removeAll(sheet);
+	emit aboutToDelete(sheet);
+	delete sheet;
 	
-	if (m_deletionSongList.isEmpty() && m_exitInProgress) {
+	if (m_deletionSheetList.isEmpty() && m_exitInProgress) {
 		QApplication::exit();
 	}
 		
@@ -453,6 +459,10 @@ int ProjectManager::rename_project_dir(const QString & olddir, const QString & n
 
 void ProjectManager::set_current_project_dir(const QString & path)
 {
+	if (path.isEmpty()) {
+		return;
+	}
+	
 	QDir newdir(path);
 	
 	config().set_property("Project", "directory", newdir.canonicalPath());
@@ -460,7 +470,7 @@ void ProjectManager::set_current_project_dir(const QString & path)
 	QStringList list = newdir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
 	m_projectDirs.clear();
 	
-	foreach(QString string, list) {
+	foreach(const QString &string, list) {
 		m_projectDirs += path + "/" + string;
 	}
 	
@@ -481,7 +491,7 @@ void ProjectManager::project_dir_rename_detected(const QString & dirname)
 	QStringList list = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
 	
 	bool startwhining = false;
-	foreach(QString string, list) {
+	foreach(const QString &string, list) {
 		if (!m_projectDirs.contains(path + "/" + string)) {
 			startwhining = true;
 			break;
