@@ -27,10 +27,11 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "AudioClip.h"
 #include "DiskIO.h"
 #include "Utils.h"
-#include "Song.h"
+#include "Sheet.h"
 #include "AudioDevice.h"
 #include <QFile>
 #include "Config.h"
+#include <limits.h>
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
@@ -142,7 +143,7 @@ QDomNode ReadSource::get_state( QDomDocument doc )
 {
 	QDomElement node = doc.createElement("Source");
 	node.setAttribute("channelcount", m_channelCount);
-	node.setAttribute("origsheetid", m_origSongId);
+	node.setAttribute("origsheetid", m_origSheetId);
 	node.setAttribute("dir", m_dir);
 	node.setAttribute("id", m_id);
 	node.setAttribute("name", m_name);
@@ -162,7 +163,7 @@ int ReadSource::set_state( const QDomNode & node )
 	
 	QDomElement e = node.toElement();
 	m_channelCount = e.attribute("channelcount", "0").toInt();
-	m_origSongId = e.attribute("origsheetid", "0").toLongLong();
+	m_origSheetId = e.attribute("origsheetid", "0").toLongLong();
 	set_dir( e.attribute("dir", "" ));
 	m_id = e.attribute("id", "").toLongLong();
 	m_rate = m_outputRate = e.attribute("rate", "0").toUInt();
@@ -175,7 +176,7 @@ int ReadSource::set_state( const QDomNode & node )
 	// For older project files, this should properly detect if the 
 	// audio source was a recording or not., in fact this should suffice
 	// and the flag wasrecording would be unneeded, but oh well....
-	if (m_origSongId != 0) {
+	if (m_origSheetId != 0) {
 		m_wasRecording = true;
 	}
 	
@@ -203,7 +204,7 @@ int ReadSource::init( )
 	}
 	
 	if (m_silent) {
-		m_length = TimeRef(LONG_LONG_MAX);
+		m_length = TimeRef(LLONG_MAX);
 		m_channelCount = 0;
 		m_origBitDepth = 16;
 		m_bufferstatus->fillStatus =  100;
@@ -221,9 +222,10 @@ int ReadSource::init( )
 	}
 	
 	m_rbReady = 0;
-	m_needSync = 0;
+	m_needSync = 1;
 	m_syncInProgress = 0;
 	m_bufferUnderRunDetected = m_wasActivated = 0;
+	m_active = 0;
 	
 	// There should be another config option for ConverterType to use for export (higher quality)
 	//converter_type = config().get_property("Conversion", "ExportResamplingConverterType", 0).toInt();
@@ -236,7 +238,7 @@ int ReadSource::init( )
 		return (m_error = COULD_NOT_OPEN_FILE);
 	}
 	
-	int converter_type = config().get_property("Conversion", "RTResamplingConverterType", 2).toInt();
+	int converter_type = config().get_property("Conversion", "RTResamplingConverterType", DEFAULT_RESAMPLE_QUALITY).toInt();
 	m_audioReader->set_converter_type(converter_type);
 	
 	set_output_rate(m_audioReader->get_file_rate());
@@ -268,7 +270,7 @@ int ReadSource::init( )
 }
 
 
-void ReadSource::set_output_rate(int rate, bool forceRate)
+void ReadSource::set_output_rate(int rate)
 {
 	Q_ASSERT(rate > 0);
 	
@@ -278,7 +280,7 @@ void ReadSource::set_output_rate(int rate, bool forceRate)
 	}
 	
 	bool useResampling = config().get_property("Conversion", "DynamicResampling", true).toBool();
-	if (useResampling || forceRate) {
+	if (useResampling) {
 		m_audioReader->set_output_rate(rate);
 	} else {
 		m_audioReader->set_output_rate(m_audioReader->get_file_rate());
@@ -328,7 +330,7 @@ void ReadSource::set_audio_clip(AudioClip* clip)
 	m_clip = clip;
 }
 
-const nframes_t ReadSource::get_nframes( ) const
+nframes_t ReadSource::get_nframes( ) const
 {
 	if (!m_audioReader) {
 		return 0;
@@ -391,7 +393,7 @@ int ReadSource::rb_read(audio_sample_t** dst, TimeRef& start, nframes_t count)
 	
 	if (start != m_rbRelativeFileReadPos) {
 		
-		TimeRef availabletime(m_buffers.at(0)->read_space(), m_outputRate);
+		TimeRef availabletime(nframes_t(m_buffers.at(0)->read_space()), m_outputRate);
 /*		printf("rb_read:: m_rbRelativeFileReadPos, start: %lld, %lld\n", m_rbRelativeFileReadPos.universal_frame(), start.universal_frame());
 		printf("rb_read:: availabletime %d\n", availabletime.to_frame(m_outputRate));*/
 		
@@ -531,6 +533,13 @@ void ReadSource::process_ringbuffer(DecodeBuffer* buffer, bool seeking)
 		}
 	}
 	
+	// Check if the resample quality has changed, it's a safe place here
+	// to reconfigure the audioreaders resample quality.
+	// This allows on the fly changing of the resample quality :)
+	if (m_diskio->get_resample_quality() != m_audioReader->get_convertor_type()) {
+		m_audioReader->set_converter_type(m_diskio->get_resample_quality());
+	}
+	
 	// Read in the samples from source
 	nframes_t toWrite = rb_file_read(buffer, toRead);
 	
@@ -619,7 +628,7 @@ void ReadSource::prepare_rt_buffers( )
 		m_buffers.append(new RingBufferNPT<float>(m_bufferSize));
 	}
 
-	TimeRef synclocation = m_clip->get_song()->get_work_location();
+	TimeRef synclocation = m_clip->get_sheet()->get_work_location();
 	start_resync(synclocation);
 }
 
@@ -632,7 +641,7 @@ BufferStatus* ReadSource::get_buffer_status()
 	int freespace = m_buffers.at(0)->write_space();
 	
 // 	printf("m_rbFileReadPos, m_length %lld, %lld\n", m_rbFileReadPos.universal_frame(), m_length.universal_frame());
-	TimeRef transport = m_clip->get_song()->get_transport_location();
+	TimeRef transport = m_clip->get_sheet()->get_transport_location();
 	TimeRef syncstartlocation = m_clip->get_track_start_location();
 	bool transportBeforeSyncStartLocation = transport < (syncstartlocation - (3 * UNIVERSAL_SAMPLE_RATE));
 	bool transportAfterClipEndLocation = transport > (m_clip->get_track_end_location() + (3 * UNIVERSAL_SAMPLE_RATE));
@@ -688,3 +697,15 @@ void ReadSource::set_diskio(DiskIO * diskio)
 	prepare_rt_buffers();
 }
 
+QString ReadSource::get_error_string() const
+{
+	switch(m_error) {
+		case COULD_NOT_OPEN_FILE: return tr("Could not open file");
+		case INVALID_CHANNEL_COUNT: return tr("Invalid channel count");
+		case ZERO_CHANNELS: return tr("File has zero channels");
+		case FILE_DOES_NOT_EXIST: return tr("The file does not exist!");
+		default: return tr("No ReadSource error set");
+	}
+	return tr("No ReadSource error set");
+}
+
