@@ -20,9 +20,10 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: AlsaDriver.cpp,v 1.19 2007/12/03 16:24:20 r_sijrier Exp $
 */
 
+/* ALSA driver based on jack-audio-connection-kit-0.109.0 alsa_driver.c */
+
 
 #include "AlsaDriver.h"
 #include "AudioChannel.h"
@@ -98,11 +99,11 @@ AlsaDriver::~AlsaDriver()
 }
 
 
-int AlsaDriver::setup(bool capture, bool playback, const QString& pcmName)
+int AlsaDriver::setup(bool capture, bool playback, const QString& pcmName, const QString& ditherShape)
 {
 	unsigned long user_nperiods = device->get_driver_property("numberofperiods", 3).toInt();
-	char *playback_pcm_name = pcmName.toAscii().data();
-	char *capture_pcm_name = pcmName.toAscii().data();
+	char *playback_pcm_name = strdup(pcmName.toAscii().data());
+	char *capture_pcm_name = strdup(pcmName.toAscii().data());
 	int shorts_first = false;
 
 	/* duplex is the default */
@@ -129,9 +130,13 @@ int AlsaDriver::setup(bool capture, bool playback, const QString& pcmName)
 	playback_frame_latency = 0;
 	channels_done = 0;
 	channels_not_done = 0;
+	dither_state = 0;
 
 	playback_addr = 0;
 	capture_addr = 0;
+	playback_interleave_skip = NULL;
+	capture_interleave_skip = NULL;
+
 
 	playback_hw_params = 0;
 	capture_hw_params = 0;
@@ -145,9 +150,20 @@ int AlsaDriver::setup(bool capture, bool playback, const QString& pcmName)
 	playback_nfds = 0;
 	capture_nfds = 0;
 
-	dither = None;
+	if (ditherShape == "Rectangular") {
+		dither = Rectangular;
+	} else if (ditherShape == "Shaped") {
+		dither = Shaped;
+	} else if (ditherShape == "Triangular") {
+		dither = Triangular;
+	} else {
+		dither = None;
+	}
+	
 	soft_mode = false;
 
+	quirk_bswap = 0;
+
 	process_count = 0;
 
 	alsa_name_playback = strdup (playback_pcm_name);
@@ -277,7 +293,7 @@ int AlsaDriver::setup(bool capture, bool playback, const QString& pcmName)
 	capture_and_playback_not_synced = false;
 
 	if (capture_handle && playback_handle) {
-		if (snd_pcm_link (capture_handle, playback_handle) != 0) {
+		if (snd_pcm_link (playback_handle, capture_handle) != 0) {
 			capture_and_playback_not_synced = true;
 		}
 	}
@@ -302,16 +318,27 @@ void AlsaDriver::release_channel_dependent_memory ()
 		free(capture_addr);
 		capture_addr = 0;
 	}
+	
+	if (playback_interleave_skip) {
+		free (playback_interleave_skip);
+		playback_interleave_skip = NULL;
+	}
+
+	if (capture_interleave_skip) {
+		free (capture_interleave_skip);
+		capture_interleave_skip = NULL;
+	}
+
 
 	if (silent) {
 		free(silent);
 		silent = 0;
 	}
 
-	/*	if (dither_state) {
-			delete dither_state;
-			dither_state = 0;
-		}*/
+	if (dither_state) {
+		free(dither_state);
+		dither_state = 0;
+	}
 }
 
 
@@ -328,21 +355,28 @@ void AlsaDriver::setup_io_function_pointers()
 		switch (dither) {
 		case Rectangular:
 			fprintf (stderr,"Rectangular dithering at 16 bits\n");
-			write_via_copy = sample_move_dither_rect_d16_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_rect_d16_sSs:
+				sample_move_dither_rect_d16_sS;
 			break;
 
 		case Triangular:
 			printf("Triangular dithering at 16 bits\n");
-			write_via_copy = sample_move_dither_tri_d16_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_tri_d16_sSs:
+				sample_move_dither_tri_d16_sS;
 			break;
 
 		case Shaped:
 			printf("Noise-shaped dithering at 16 bits\n");
-			write_via_copy = sample_move_dither_shaped_d16_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_shaped_d16_sSs:
+				sample_move_dither_shaped_d16_sS;
 			break;
 
 		default:
-			write_via_copy = sample_move_d16_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_d16_sSs : sample_move_d16_sS;
 			break;
 		}
 		break;
@@ -357,22 +391,29 @@ void AlsaDriver::setup_io_function_pointers()
 		switch (dither) {
 		case Rectangular:
 			printf("Rectangular dithering at 24 bits\n");
-			write_via_copy = sample_move_dither_rect_d24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_rect_d24_sSs:
+				sample_move_dither_rect_d24_sS;
 			break;
 
 		case Triangular:
 			printf("Triangular dithering at 24 bits\n");
-			write_via_copy = sample_move_dither_tri_d24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_tri_d24_sSs:
+				sample_move_dither_tri_d24_sS;
 			break;
 
 		case Shaped:
 			printf("Noise-shaped dithering at 24 bits\n");
-			write_via_copy = sample_move_dither_shaped_d24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_shaped_d24_sSs:
+				sample_move_dither_shaped_d24_sS;
 			break;
 
-		default:
-			write_via_copy = sample_move_d24_sS;
-			break;
+			default:
+			write_via_copy = quirk_bswap?
+				sample_move_d24_sSs : sample_move_d24_sS;
+				break;
 		}
 		break;
 
@@ -386,21 +427,28 @@ void AlsaDriver::setup_io_function_pointers()
 		switch (dither) {
 		case Rectangular:
 			printf("Rectangular dithering at 32 bits\n");
-			write_via_copy = sample_move_dither_rect_d32u24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_rect_d32u24_sSs:
+				sample_move_dither_rect_d32u24_sS;
 			break;
 
 		case Triangular:
-			printf("Triangular dithering at 32 bits\n");
-			write_via_copy = sample_move_dither_tri_d32u24_sS;
+			printf("Triangular dithering at 16 bits\n");
+			write_via_copy = quirk_bswap?
+				sample_move_dither_tri_d32u24_sSs:
+				sample_move_dither_tri_d32u24_sS;
 			break;
 
 		case Shaped:
 			printf("Noise-shaped dithering at 32 bits\n");
-			write_via_copy = sample_move_dither_shaped_d32u24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_dither_shaped_d32u24_sSs:
+				sample_move_dither_shaped_d32u24_sS;
 			break;
 
 		default:
-			write_via_copy = sample_move_d32u24_sS;
+			write_via_copy = quirk_bswap?
+				sample_move_d32u24_sSs : sample_move_d32u24_sS;
 			break;
 		}
 		break;
@@ -408,13 +456,16 @@ void AlsaDriver::setup_io_function_pointers()
 
 	switch (capture_sample_bytes) {
 	case 2:
-		read_via_copy = sample_move_dS_s16;
+		read_via_copy = quirk_bswap?
+			sample_move_dS_s16s : sample_move_dS_s16;
 		break;
 	case 3:
-		read_via_copy = sample_move_dS_s24;
+		read_via_copy = quirk_bswap?
+			sample_move_dS_s24s : sample_move_dS_s24;
 		break;
 	case 4:
-		read_via_copy = sample_move_dS_s32u24;
+		read_via_copy = quirk_bswap?
+			sample_move_dS_s32u24s : sample_move_dS_s32u24;
 		break;
 	}
 }
@@ -431,57 +482,64 @@ int AlsaDriver::configure_stream(char *device_name,
 	int err, format;
 	snd_pcm_uframes_t stop_th;
 	static struct {
-		char Name[16];
+		char Name[32];
 		snd_pcm_format_t format;
+		int swapped;
 		int bitdepth;
-	}
-	formats[] = {
-			{"32bit", SND_PCM_FORMAT_S32, 32},
-			{"24bit", SND_PCM_FORMAT_S24_3, 24},
-			{"16bit", SND_PCM_FORMAT_S16, 16},
-		};
-#define NOFORMATS (sizeof(formats)/sizeof(formats[0]))
+	} formats[] = {
+		{"32bit little-endian", SND_PCM_FORMAT_S32_LE, IS_LE, 32},
+		{"32bit big-endian", SND_PCM_FORMAT_S32_BE, IS_BE, 32},
+		{"24bit little-endian", SND_PCM_FORMAT_S24_3LE, IS_LE, 24},
+		{"24bit big-endian", SND_PCM_FORMAT_S24_3BE, IS_BE, 24},
+		{"16bit little-endian", SND_PCM_FORMAT_S16_LE, IS_LE, 16},
+		{"16bit big-endian", SND_PCM_FORMAT_S16_BE, IS_BE, 16},
+	};
+#define NUMFORMATS (sizeof(formats)/sizeof(formats[0]))
+#define FIRST_16BIT_FORMAT 4
 
 	if ((err = snd_pcm_hw_params_any (handle, hw_params)) < 0)  {
-		PERROR ("ALSA: no playback configurations available (%s)", snd_strerror (err));
+		printf("ALSA: no playback configurations available (%s)\n", snd_strerror (err));
 		return -1;
 	}
 
 	if ((err = snd_pcm_hw_params_set_periods_integer (handle, hw_params))  < 0) {
-		qDebug ("ALSA: cannot restrict period size to integral value.");
+		printf("ALSA: cannot restrict period size to integral value.\n");
 		return -1;
 	}
 
 	if ((err = snd_pcm_hw_params_set_access (handle, hw_params, SND_PCM_ACCESS_MMAP_NONINTERLEAVED)) < 0) {
-		if ((err = snd_pcm_hw_params_set_access ( handle, hw_params, SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0) {
-			qDebug ("ALSA: mmap-based access is not possible for the %s "
-				"stream of this audio interface", stream_name);
+		if ((err = snd_pcm_hw_params_set_access (handle, hw_params, SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0) {
+			if ((err = snd_pcm_hw_params_set_access (handle, hw_params, SND_PCM_ACCESS_MMAP_COMPLEX)) < 0) {
+				printf("ALSA: mmap-based access is not possible for the %s "
+					"stream of this audio interface\n", stream_name);
 			return -1;
+			}
 		}
 	}
 
-	format = (sample_width == 4) ? 0 : (NOFORMATS - 1);
+	format = (sample_width == 4) ? 0 : (NUMFORMATS - 1);
 	while (1) {
 		if ((err = snd_pcm_hw_params_set_format ( handle, hw_params, formats[format].format)) < 0) {
-			int failed_format = format;
-			if (( (sample_width == 4) ? (format++ < (int)NOFORMATS - 1) : (format-- > 0))) {
-				fprintf (stderr,"Note: audio device %s doesn't support a %s sample format"
-					" so Traverso will try a %s format instead\n", device_name,
-					formats[failed_format].Name,
-					formats[format].Name);
-			} else {
-				PERROR ("ALSA Driver: Sorry. The audio interface \"%s\" doesn't support any of the"
-					" hardware sample formats that Traverso's alsa-driver can use.", device_name);
+			
+			if (( (sample_width == 4) ? (format++ >= int(NUMFORMATS) - 1) : (format-- <= 0))) {
+				printf("ALSA Driver: Sorry. The audio interface \"%s\" doesn't support any of the"
+					" hardware sample formats that Traverso's alsa-driver can use.\n", device_name);
 				return -1;
 			}
 		} else {
+			if (formats[format].swapped) {
+				quirk_bswap = 1;
+			} else {
+				quirk_bswap = 0;
+			}
+			printf("ALSA: final selected sample format for %s: %s\n", stream_name, formats[format].Name);
 			device->set_bit_depth(formats[format].bitdepth);
 			break;
 		}
 	}
 
 	if ( (err = snd_pcm_hw_params_set_rate_near (handle, hw_params, &frame_rate, NULL)) < 0) {
-		PERROR ("ALSA: cannot set sample/frame rate to % for %s", (double)frame_rate, stream_name);
+		printf("ALSA: cannot set sample/frame rate to % for %s\n", (double)frame_rate, stream_name);
 		return -1;
 	}
 
@@ -513,12 +571,12 @@ int AlsaDriver::configure_stream(char *device_name,
 	}
 
 	if ((err = snd_pcm_hw_params_set_channels (handle, hw_params, *nchns)) < 0) {
-		qWarning ("ALSA: cannot set channel count to %lu for %s", *nchns, stream_name);
+		printf("ALSA: cannot set channel count to %lu for %s\n", *nchns, stream_name);
 		return -1;
 	}
 	int frperscycle = frames_per_cycle;
 	if ((err = snd_pcm_hw_params_set_period_size (handle, hw_params, frames_per_cycle, 0)) < 0) {
-		qWarning ("ALSA: cannot set period size to %d frames for %s", frperscycle, stream_name);
+		printf("ALSA: cannot set period size to %d frames for %s\n", frperscycle, stream_name);
 		return -1;
 	}
 
@@ -528,29 +586,27 @@ int AlsaDriver::configure_stream(char *device_name,
 		*nperiodsp = user_nperiods;
 
 	if (snd_pcm_hw_params_set_periods_near (handle, hw_params, nperiodsp, NULL) < 0) {
-		PERROR ("ALSA: cannot set number of periods to %u for %s", *nperiodsp, stream_name);
+		printf("ALSA: cannot set number of periods to %u for %s\n", *nperiodsp, stream_name);
 		return -1;
 	}
 
 	if (*nperiodsp < user_nperiods) {
-		PERROR ("ALSA: got smaller periods %u than %u for %s",
-			*nperiodsp, (unsigned int) user_nperiods,
-			stream_name);
+		printf("ALSA: use %d periods for %s\n", *nperiodsp, stream_name);
 		return -1;
 	}
 
 	if (!is_power_of_two(frames_per_cycle)) {
-		PERROR("Traverso: frames must be a power of two (64, 512, 1024, ...)");
+		printf("Traverso: frames must be a power of two (64, 512, 1024, ...)\n");
 		return -1;
 	}
 
 	if ((err = snd_pcm_hw_params_set_buffer_size (handle, hw_params,  *nperiodsp * frames_per_cycle)) < 0) {
-		PERROR ("ALSA: cannot set buffer length to %d for %s", *nperiodsp * frames_per_cycle, stream_name);
+		printf("ALSA: cannot set buffer length to %d for %s\n", *nperiodsp * frames_per_cycle, stream_name);
 		return -1;
 	}
 
 	if ((err = snd_pcm_hw_params (handle, hw_params)) < 0) {
-		PERROR ("ALSA: cannot set hardware parameters for %s", stream_name);
+		printf("ALSA: cannot set hardware parameters for %s\n", stream_name);
 		device->message(tr("ALSA Driver: Unable to configure hardware, is it in use by another application?"), AudioDevice::WARNING);
 		return -1;
 	}
@@ -558,7 +614,7 @@ int AlsaDriver::configure_stream(char *device_name,
 	snd_pcm_sw_params_current (handle, sw_params);
 
 	if ((err = snd_pcm_sw_params_set_start_threshold (handle, sw_params, 0U)) < 0) {
-		PERROR ("ALSA: cannot set start mode for %s", stream_name);
+		printf("ALSA: cannot set start mode for %s\n", stream_name);
 		return -1;
 	}
 
@@ -568,12 +624,12 @@ int AlsaDriver::configure_stream(char *device_name,
 	}
 
 	if ((err = snd_pcm_sw_params_set_stop_threshold (handle, sw_params, stop_th)) < 0) {
-		PERROR ("ALSA: cannot set stop mode for %s", stream_name);
+		printf("ALSA: cannot set stop mode for %s\n", stream_name);
 		return -1;
 	}
 
 	if ((err = snd_pcm_sw_params_set_silence_threshold (handle, sw_params, 0)) < 0) {
-		PERROR ("ALSA: cannot set silence threshold for %s", stream_name);
+		printf("ALSA: cannot set silence threshold for %s\n", stream_name);
 		return -1;
 	}
 
@@ -597,13 +653,12 @@ int AlsaDriver::configure_stream(char *device_name,
 		err = snd_pcm_sw_params_set_avail_min (handle, sw_params, frames_per_cycle);
 
 	if (err < 0) {
-		PERROR ("ALSA: cannot set avail min for %s", stream_name);
+		printf("ALSA: cannot set avail min for %s\n", stream_name);
 		return -1;
 	}
 
 	if ((err = snd_pcm_sw_params (handle, sw_params)) < 0) {
-		PERROR ("ALSA: cannot set software parameters for %s\n",
-			stream_name);
+		printf("ALSA: cannot set software parameters for %s\n",	stream_name);
 		return -1;
 	}
 
@@ -626,8 +681,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 	frames_per_cycle = frames_per_interupt;
 	user_nperiods = nperiods;
 
-	fprintf (stderr, "configuring for %d Hz, period = %ld frames, buffer = %ld periods\n", rate, (long)frames_per_cycle, user_nperiods);
-
+	fprintf (stderr, "configuring for %d Hz, period=%ld frames (%.1f ms), buffer=%ld periods\n", 
+		 rate, (long)frames_per_cycle,(((float)frames_per_cycle / (float) rate) * 1000.0f), user_nperiods);
 	if (capture_handle) {
 		if (configure_stream (
 					alsa_name_capture,
@@ -700,7 +755,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 		err = snd_pcm_hw_params_get_period_size (playback_hw_params, &p_period_size, &dir);
 		err = snd_pcm_hw_params_get_format (playback_hw_params,	&playback_sample_format);
 		err = snd_pcm_hw_params_get_access (playback_hw_params, &access);
-		playback_interleaved = (access == SND_PCM_ACCESS_MMAP_INTERLEAVED);
+		playback_interleaved = (access == SND_PCM_ACCESS_MMAP_INTERLEAVED) 
+					|| (access == SND_PCM_ACCESS_MMAP_COMPLEX);
 
 		if (p_period_size != frames_per_cycle) {
 			PERROR ("alsa_pcm: requested an interrupt every %ld frames but got %ld frames for playback", (long)frames_per_cycle, p_period_size);
@@ -714,7 +770,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 		err = snd_pcm_hw_params_get_period_size (capture_hw_params, &c_period_size, &dir);
 		err = snd_pcm_hw_params_get_format (capture_hw_params, &(capture_sample_format));
 		err = snd_pcm_hw_params_get_access (capture_hw_params, &access);
-		capture_interleaved = (access == SND_PCM_ACCESS_MMAP_INTERLEAVED);
+		capture_interleaved = (access == SND_PCM_ACCESS_MMAP_INTERLEAVED) 
+					|| (access == SND_PCM_ACCESS_MMAP_COMPLEX);
 
 
 		if (c_period_size != frames_per_cycle) {
@@ -729,7 +786,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 	if (playback_handle) {
 		switch (playback_sample_format) {
 		case SND_PCM_FORMAT_S32_LE:
-		case SND_PCM_FORMAT_S24_3:
+		case SND_PCM_FORMAT_S24_3LE:
+		case SND_PCM_FORMAT_S24_3BE:
 		case SND_PCM_FORMAT_S16_LE:
 		case SND_PCM_FORMAT_S32_BE:
 		case SND_PCM_FORMAT_S16_BE:
@@ -744,7 +802,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 	if (capture_handle) {
 		switch (capture_sample_format) {
 		case SND_PCM_FORMAT_S32_LE:
-		case SND_PCM_FORMAT_S24_3:
+		case SND_PCM_FORMAT_S24_3LE:
+		case SND_PCM_FORMAT_S24_3BE:
 		case SND_PCM_FORMAT_S16_LE:
 		case SND_PCM_FORMAT_S32_BE:
 		case SND_PCM_FORMAT_S16_BE:
@@ -763,11 +822,10 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 			PERROR ("ALSA: %s: mmap areas info error", alsa_name_playback);
 			return -1;
 		}
-		playback_interleave_skip = my_areas[0].step / 8;
+		
 		interleave_unit = snd_pcm_format_physical_width(playback_sample_format) / 8;
 	} else {
 		interleave_unit = 0;  /* NOT USED */
-		playback_interleave_skip = snd_pcm_format_physical_width(playback_sample_format) / 8;
 	}
 
 	if (capture_interleaved) {
@@ -777,9 +835,6 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 			PERROR ("ALSA: %s: mmap areas info error", alsa_name_capture);
 			return -1;
 		}
-		capture_interleave_skip = my_areas[0].step / 8;
-	} else {
-		capture_interleave_skip = snd_pcm_format_physical_width(capture_sample_format) / 8;
 	}
 
 	if (playback_nchannels > capture_nchannels) {
@@ -807,6 +862,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 	if (playback_handle) {
 		playback_addr =  (char**) malloc (sizeof (char *) * playback_nchannels);
 		memset (playback_addr, 0, sizeof (char *) * playback_nchannels);
+		playback_interleave_skip = (unsigned long *) malloc (sizeof (unsigned long *) * playback_nchannels);
+		memset (playback_interleave_skip, 0, sizeof (unsigned long *) * playback_nchannels);
 		silent = (unsigned long *) malloc (sizeof (unsigned long) * playback_nchannels);
 
 		for (chn = 0; chn < playback_nchannels; chn++) {
@@ -823,6 +880,8 @@ int  AlsaDriver::set_parameters (nframes_t frames_per_interupt,
 	if (capture_handle) {
 		capture_addr = (char **) malloc (sizeof (char *) * capture_nchannels);
 		memset (capture_addr, 0, sizeof (char *) * capture_nchannels);
+		capture_interleave_skip = (unsigned long *) malloc (sizeof (unsigned long *) * capture_nchannels);
+		memset (capture_interleave_skip, 0, sizeof (unsigned long *) * capture_nchannels);
 	}
 
 	period_usecs = (trav_time_t) floor ((((float) frames_per_cycle) / frame_rate) * 1000000.0f);
@@ -859,6 +918,7 @@ int AlsaDriver::get_channel_addresses (snd_pcm_uframes_t *capture_avail,
 		for (chn = 0; chn < capture_nchannels; chn++) {
 			const snd_pcm_channel_area_t *a = &capture_areas[chn];
 			capture_addr[chn] = (char *) a->addr + ((a->first + a->step * *capture_offset) / 8);
+			capture_interleave_skip[chn] = (unsigned long ) (a->step / 8);
 		}
 	}
 
@@ -871,6 +931,7 @@ int AlsaDriver::get_channel_addresses (snd_pcm_uframes_t *capture_avail,
 		for (chn = 0; chn < playback_nchannels; chn++) {
 			const snd_pcm_channel_area_t *a = &playback_areas[chn];
 			playback_addr[chn] = (char *) a->addr + ((a->first + a->step * *playback_offset) / 8);
+			playback_interleave_skip[chn] = (unsigned long ) (a->step / 8);
 		}
 	}
 
@@ -1087,9 +1148,10 @@ again:
 
 	while (need_playback || need_capture) {
 
-		unsigned int p_timed_out, c_timed_out;
+		int poll_result;
 		unsigned int ci = 0;
 		unsigned int nfds;
+		unsigned short revents;
 
 		nfds = 0;
 
@@ -1130,7 +1192,8 @@ again:
 
 		device->transport_cycle_end(poll_enter);
 
-		if (poll (pfd, nfds, poll_timeout) < 0) {
+		poll_result = poll (pfd, nfds, poll_timeout);
+		if (poll_result < 0) {
 
 			if (errno == EINTR) {
 				printf ("poll interrupt\n");
@@ -1185,29 +1248,20 @@ again:
 			return (pfd[nfds-1].revents == POLLIN) ? 0 : -1;
 		}
 
-		p_timed_out = 0;
-
 		if (need_playback) {
-			for (i = 0; i < playback_nfds; i++) {
-				if (pfd[i].revents & POLLERR) {
-					PWARN("playback pollerror, xrun_detected is true");
-					xrun_detected = true;
-				}
-
-				if (pfd[i].revents == 0) {
-					p_timed_out++;
-#ifdef DEBUG_WAKEUP
-
-					fprintf (stderr, "%" PRIu64
-						" playback stream timed out\n",
-						poll_ret);
-#endif
-
-				}
+			if (snd_pcm_poll_descriptors_revents(playback_handle, &pfd[0], playback_nfds, &revents) < 0) {
+				qWarning("ALSA: playback revents failed");
+				*status = -6;
+				return 0;
 			}
 
-			if (p_timed_out == 0) {
+			if (revents & POLLERR) {
+				xrun_detected = TRUE;
+			}
+
+			if (revents & POLLOUT) {
 				need_playback = 0;
+
 #ifdef DEBUG_WAKEUP
 
 				fprintf (stderr, "%" PRIu64
@@ -1218,28 +1272,18 @@ again:
 			}
 		}
 
-		c_timed_out = 0;
-
 		if (need_capture) {
-			for (i = ci; i < nfds; i++) {
-				if (pfd[i].revents & POLLERR) {
-					PWARN("capture pollerror, xrun_detected is true");
-					xrun_detected = true;
-				}
-
-				if (pfd[i].revents == 0) {
-					c_timed_out++;
-#ifdef DEBUG_WAKEUP
-
-					fprintf (stderr, "%" PRIu64
-						" capture stream timed out\n",
-						poll_ret);
-#endif
-
-				}
+			if (snd_pcm_poll_descriptors_revents(capture_handle, &pfd[ci], capture_nfds, &revents) < 0) {
+				qWarning ("ALSA: capture revents failed");
+				*status = -6;
+				return 0;
 			}
 
-			if (c_timed_out == 0) {
+			if (revents & POLLERR) {
+				xrun_detected = TRUE;
+			}
+
+			if (revents & POLLIN) {
 				need_capture = 0;
 #ifdef DEBUG_WAKEUP
 
@@ -1251,9 +1295,8 @@ again:
 			}
 		}
 
-		if ((p_timed_out && (p_timed_out == playback_nfds)) &&
-				(c_timed_out && (c_timed_out == capture_nfds))) {
-			PERROR ("ALSA: poll time out, polled for %ld usecs", (long)(poll_ret - poll_enter));
+		if (poll_result == 0) {
+			qWarning ("ALSA: poll time out, polled for %ld usecs", (long)(poll_ret - poll_enter));
 			*status = -5;
 			return 0;
 		}
@@ -1390,13 +1433,13 @@ int AlsaDriver::_read(nframes_t nframes)
 	audio_sample_t* buf;
 	int err;
 
-	if (!capture_handle) {
-		return 0;
-	}
-
 	if (nframes > frames_per_cycle) {
 		return -1;
 	}
+	
+	if (!capture_handle) {
+		return 0;
+	}
 
 	nread = 0;
 	contiguous = 0;
@@ -1583,7 +1626,7 @@ QString AlsaDriver::alsa_device_name(bool longname, int devicenumber)
 	snd_ctl_card_info_alloca(&info);
 
 	if ((err = snd_ctl_open(&handle, name, devicenumber)) < 0) {
-		PMESG("Control open (%i): %s", devicenumber, snd_strerror(err));
+		PMESG("AlsaDriver::alsa_device_name: Control open (device %i): %s", devicenumber, snd_strerror(err));
 		return "";
 	}
 
