@@ -38,7 +38,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
 class AudioDeviceThread;
 class Driver;
-class Client;
+class TAudioDeviceClient;
 class AudioChannel;
 class AudioBus;
 #if defined (JACK_SUPPORT)
@@ -63,12 +63,12 @@ public:
 				const QString& device="hw:0",
 			   	const QString& ditherShape="None");
 
-	void add_client(Client* client);
-	void remove_client(Client* client);
+	void add_client(TAudioDeviceClient* client);
+	void remove_client(TAudioDeviceClient* client);
 	
-	void transport_start(Client* client);
-	void transport_stop(Client* client);
-	int transport_seek_to(Client* client, TimeRef location);
+	void transport_start(TAudioDeviceClient* client);
+	void transport_stop(TAudioDeviceClient* client);
+	int transport_seek_to(TAudioDeviceClient* client, TimeRef location);
 
 	/**
 	 * Get the Playback AudioBus instance with name \a name.
@@ -254,15 +254,15 @@ signals:
 	 *        This signal will be emited after succesfull Client removal from within the GUI Thread!
 	 * @param  The Client \a client which as been removed from the AudioDevice
 	 */
-	void clientRemoved(Client*);
+	void clientRemoved(TAudioDeviceClient*);
 	
 	void xrunStormDetected();
 	
 	void message(QString, int);
 
 private slots:
-	void private_add_client(Client* client);
-	void private_remove_client(Client* client);
+	void private_add_client(TAudioDeviceClient* client);
+	void private_remove_client(TAudioDeviceClient* client);
 	void audiothread_finished();
 	void switch_to_null_driver();
 	void reset_xrun_counter() {m_xrunCount = 0;}
