@@ -24,10 +24,12 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "Plugin.h"
 #include "PluginManager.h"
 #include <Tsar.h>
-#include <InputEngine.h>
 #include <Sheet.h>
 #include <AddRemove.h>
 #include "AudioBus.h"
+#include "GainEnvelope.h"
+#include "Curve.h"
+#include "Mixer.h"
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
@@ -42,8 +44,8 @@ PluginChain::PluginChain(ContextItem * parent)
 PluginChain::PluginChain(ContextItem* parent, Sheet* sheet)
 	: ContextItem(parent)
 {
-	m_fader = new GainEnvelope(sheet);
-	set_sheet(sheet);
+        m_fader = new GainEnvelope(sheet);
+        set_sheet(sheet);
 }
 
 PluginChain::~ PluginChain()
@@ -99,7 +101,7 @@ Command* PluginChain::add_plugin(Plugin * plugin, bool historable)
 {
 	plugin->set_history_stack(get_history_stack());
 	
-	return new AddRemove( this, plugin, historable, m_sheet,
+        return new AddRemove( this, plugin, historable, m_sheet,
 		"private_add_plugin(Plugin*)", "pluginAdded(Plugin*)",
 		"private_remove_plugin(Plugin*)", "pluginRemoved(Plugin*)",
 		tr("Add Plugin (%1)").arg(plugin->get_name()));
@@ -108,7 +110,7 @@ Command* PluginChain::add_plugin(Plugin * plugin, bool historable)
 
 Command* PluginChain::remove_plugin(Plugin* plugin, bool historable)
 {
-	return new AddRemove( this, plugin, historable, m_sheet,
+        return new AddRemove( this, plugin, historable, m_sheet,
 		"private_remove_plugin(Plugin*)", "pluginRemoved(Plugin*)",
 		"private_add_plugin(Plugin*)", "pluginAdded(Plugin*)",
 		tr("Remove Plugin (%1)").arg(plugin->get_name()));
@@ -134,8 +136,8 @@ void PluginChain::private_remove_plugin( Plugin * plugin )
 
 void PluginChain::set_sheet(Sheet * sheet)
 {
-	m_sheet = sheet;
-	set_history_stack(m_sheet->get_history_stack());
-	m_fader->set_sheet(sheet);
+        m_sheet = sheet;
+        set_history_stack(m_sheet->get_history_stack());
+        m_fader->set_sheet(sheet);
 }
 
