@@ -539,9 +539,9 @@ void AudioClipView::draw_db_lines(QPainter* p, qreal xstart, int pixelcount)
 	int linestartpos = xstart;
 	if (xstart < m_lineOffset) linestartpos = m_lineOffset;
 
-	if (m_mergedView) {
-		channels = 1;
-	}
+    if ((m_mergedView) || (channels == 0)) {
+                     channels = 1;
+             }
 
 	// calculate the height of one channel
 	if (m_height >= m_mimimumheightforinfoarea) {
