@@ -49,7 +49,10 @@ class SheetView;
 class ViewItem : public ContextItem, public QGraphicsItem
 {
 	Q_OBJECT
-	
+#if QT_VERSION >= 0x040600
+        Q_INTERFACES(QGraphicsItem)
+#endif
+
 public:
 
 	ViewItem(ViewItem* parentViewItem=0, ContextItem* parentContext=0);
