@@ -22,7 +22,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "../config.h"
 
 #include <libtraversocore.h>
-#include "libtraversosongcanvas.h"
+#include "libtraversosheetcanvas.h"
 #include <AudioDevice.h> 
 
 #include <QDockWidget>
@@ -34,31 +34,35 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include <QDesktopServices>
 #include <QTextStream>
 #include <QStackedWidget>
+#include <QFileDialog>
+#include <QFileInfo>
 
 #include "Interface.h"
-#include "BusMonitor.h"
 #include "ProjectManager.h"
 #include "ViewPort.h"
 #include "FadeCurve.h"
 #include "Config.h"
 #include "Plugin.h"
-#include "CorrelationMeterWidget.h"
-#include "SpectralMeterWidget.h"
-		
 #include "Import.h"
+#include "TimeLine.h"
+#include "AudioFileCopyConvert.h"
 
-#include "songcanvas/SongWidget.h"
+#include "../sheetcanvas/SheetWidget.h"
 
 #include "ui_QuickStart.h"
 
+#include "widgets/BusMonitor.h"
 #include "widgets/InfoWidgets.h"
 #include "widgets/ResourcesWidget.h"
+#include "widgets/CorrelationMeterWidget.h"
+#include "widgets/SpectralMeterWidget.h"
+#include "widgets/TransportConsoleWidget.h"
 
 #include "dialogs/settings/SettingsDialog.h"
 #include "dialogs/project/ProjectManagerDialog.h"
 #include "dialogs/project/OpenProjectDialog.h"
 #include "dialogs/project/NewProjectDialog.h"
-#include "dialogs/project/NewSongDialog.h"
+#include "dialogs/project/NewSheetDialog.h"
 #include "dialogs/project/NewTrackDialog.h"
 #include "dialogs/MarkerDialog.h"
 #include "dialogs/BusSelectorDialog.h"
@@ -67,6 +71,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "dialogs/ProjectConverterDialog.h"
 #include "dialogs/ExportDialog.h"
 #include "dialogs/CDWritingDialog.h"
+#include "dialogs/project/ImportClipsDialog.h"
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
@@ -159,15 +164,42 @@ Interface::Interface()
 	busMonitorDW->setWidget(busMonitor);
 	addDockWidget(Qt::RightDockWidgetArea, busMonitorDW);
 	
-	m_infoBar = new InfoToolBar(this);
-	addToolBar(m_infoBar);
-	
 	m_sysinfo = new SysInfoToolBar(this);
+	m_sysinfo->setObjectName("System Info Toolbar");
 	addToolBar(Qt::BottomToolBarArea, m_sysinfo);
 	
+	m_progressBar = new ProgressToolBar(this);
+	m_progressBar->setObjectName("Progress Toolbar");
+	addToolBar(Qt::BottomToolBarArea, m_progressBar);
+	m_progressBar->hide();
+
+	m_projectToolBar = new QToolBar(this);
+	m_projectToolBar->setObjectName("Project Toolbar");
+	addToolBar(m_projectToolBar);
 	
+	m_editToolBar = new QToolBar(this);
+	m_editToolBar->setObjectName("Edit Toolbar");
+	addToolBar(m_editToolBar);
+	
+	transportConsole = new TransportConsoleWidget(this);
+	transportConsole->setObjectName("Transport Console");
+#if defined (Q_WS_MAC)
+	addToolBar(Qt::BottomToolBarArea, transportConsole);
+#else
+	addToolBar(Qt::TopToolBarArea, transportConsole);
+#endif
+
+	if (config().get_property("Themer", "textundericons", false).toBool()) {
+		m_projectToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+		m_editToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+	}
+
+	int iconsize = config().get_property("Themer", "iconsize", "22").toInt();
+	m_projectToolBar->setIconSize(QSize(iconsize, iconsize));
+	m_editToolBar->setIconSize(QSize(iconsize, iconsize));
+
 	// Some default values.
-	currentSongWidget = 0;
+	currentSheetWidget = 0;
 	m_exportDialog = 0;
 	m_cdWritingDialog = 0;
 	m_settingsdialog = 0;
@@ -175,9 +207,8 @@ Interface::Interface()
 	m_openProjectDialog = 0;
 	m_newProjectDialog = 0;
 	m_insertSilenceDialog = 0;
-	m_markerDialog = 0;
 	m_busSelector = 0;
-	m_newSongDialog = 0;
+	m_newSheetDialog = 0;
 	m_newTrackDialog = 0;
 	m_quickStart = 0;
 	m_restoreProjectBackupDialog = 0;
@@ -192,7 +223,7 @@ Interface::Interface()
 
 	// Connections to core:
 	connect(&pm(), SIGNAL(projectLoaded(Project*)), this, SLOT(set_project(Project*)));
-	connect(&pm(), SIGNAL(aboutToDelete(Song*)), this, SLOT(delete_songwidget(Song*)));
+	connect(&pm(), SIGNAL(aboutToDelete(Sheet*)), this, SLOT(delete_sheetwidget(Sheet*)));
 	connect(&pm(), SIGNAL(unsupportedProjectDirChangeDetected()), this, SLOT(project_dir_change_detected()));	
 	connect(&pm(), SIGNAL(projectLoadFailed(QString,QString)), this, SLOT(project_load_failed(QString,QString)));
 	connect(&pm(), SIGNAL(projectFileVersionMismatch(QString,QString)), this, SLOT(project_file_mismatch(QString,QString)), Qt::QueuedConnection);
@@ -200,6 +231,10 @@ Interface::Interface()
 	cpointer().add_contextitem(this);
 
 	connect(&config(), SIGNAL(configChanged()), this, SLOT(config_changed()));
+	connect(&config(), SIGNAL(configChanged()), this, SLOT(update_follow_state()));
+	update_follow_state();
+
+	setUnifiedTitleAndToolBarOnMac(true);
 }
 
 Interface::~Interface()
@@ -221,92 +256,106 @@ void Interface::set_project(Project* project)
 {
 	PENTER;
 	
+	m_project = project;
+
 	if ( project ) {
-		connect(project, SIGNAL(currentSongChanged(Song*)), this, SLOT(show_song(Song*)));
+		connect(project, SIGNAL(currentSheetChanged(Sheet*)), this, SLOT(show_sheet(Sheet*)));
+		connect(project, SIGNAL(projectLoadFinished()), this, SLOT(sheet_selector_update_sheets()));
+		connect(m_project, SIGNAL(sheetAdded(Sheet*)), this, SLOT(sheet_selector_sheet_added(Sheet*)));
+		connect(m_project, SIGNAL(sheetRemoved(Sheet*)), this, SLOT(sheet_selector_sheet_removed(Sheet*)));
 		setWindowTitle(project->get_title() + " - Traverso");
 		m_projectSaveAction->setEnabled(true);
-		m_projectSongManagerAction->setEnabled(true);
+		m_projectSheetManagerAction->setEnabled(true);
 		m_projectExportAction->setEnabled(true);
-		m_songMenuAction->setEnabled(true);
+		m_sheetMenuAction->setEnabled(true);
 
-		// the project's songs will be deleted _after_
+		// the project's sheets will be deleted _after_
 		// the project has been deleted, which will happen after this
-		// function returns. When the songs have been disconnected from the
-		// audiodevice, delete_songwidget(Song* song) is called for all the songs
-		// in the project. Meanwhile, disable updates of the SongWidgets (and implicitily
+		// function returns. When the sheets have been disconnected from the
+		// audiodevice, delete_sheetwidget(Sheet* sheet) is called for all the sheets
+		// in the project. Meanwhile, disable updates of the SheetWidgets (and implicitily
 		// all their childrens) to avoid the (unlikely) situation of a paint event that 
 		// refers to data that was part of the then deleted project!
-		// The reason to not delete the SongWidgets right now is that the newly loaded project
-		// now will be able to create and show it's songcanvas first, which improves the 
+		// The reason to not delete the SheetWidgets right now is that the newly loaded project
+		// now will be able to create and show it's sheetcanvas first, which improves the 
 		// users experience a lot!
-		foreach(SongWidget* sw, m_songWidgets) {
+		foreach(SheetWidget* sw, m_sheetWidgets) {
 			sw->setUpdatesEnabled(false);
 		}
 	} else {
-		if (pm().exit_in_progress()) {
-			hide();
-		}
 		m_projectSaveAction->setEnabled(false);
-		m_projectSongManagerAction->setEnabled(false);
+		m_projectSheetManagerAction->setEnabled(false);
 		m_projectExportAction->setEnabled(false);
-		m_songMenuAction->setEnabled(false);
+		m_sheetMenuAction->setEnabled(false);
 		setWindowTitle("Traverso");
 		// No project loaded, the currently  loaded project will be deleted after this
-		// function returns, if the songcanvas is still painting (due playback e.g.) we
+		// function returns, if the sheetcanvas is still painting (due playback e.g.) we
 		// could get a crash due canvas items refering to data that was managed by the project.
-		// so let's delete the SongWidgets before the project is deleted!
-		if (m_songWidgets.contains(0)) {
-			delete m_songWidgets.take(0);
+		// so let's delete the SheetWidgets before the project is deleted!
+		if (m_sheetWidgets.contains(0)) {
+			delete m_sheetWidgets.take(0);
 		}
-		foreach(SongWidget* sw, m_songWidgets) {
-			delete_songwidget(sw->get_song());
+		foreach(SheetWidget* sw, m_sheetWidgets) {
+			delete_sheetwidget(sw->get_sheet());
 		}
 	}
 }
 
-void Interface::delete_songwidget(Song* song)
+void Interface::delete_sheetwidget(Sheet* sheet)
 {
-	SongWidget* sw = m_songWidgets.value(song);
+	SheetWidget* sw = m_sheetWidgets.value(sheet);
 	if (sw) {
-		m_songWidgets.remove(song);
+		m_sheetWidgets.remove(sheet);
 		centerAreaWidget->removeWidget(sw);
 		delete sw;
 	}
 }
 
 
-void Interface::show_song(Song* song)
+void Interface::show_sheet(Sheet* sheet)
 {
 	PENTER;
 	
-	SongWidget* songWidget = 0;
+	SheetWidget* sheetWidget = 0;
 	
-	if (!song) {
+	if (!sheet) {
 		Project* project = pm().get_project();
-		if (project && project->get_songs().size() == 0) {
-			songWidget = m_songWidgets.value(0);
+		if (project && project->get_sheets().size() == 0) {
+			sheetWidget = m_sheetWidgets.value(0);
 			
-			if (!songWidget) {
-				songWidget = new SongWidget(0, centerAreaWidget);
-				centerAreaWidget->addWidget(songWidget);
-				m_songWidgets.insert(0, songWidget);
+			if (!sheetWidget) {
+				sheetWidget = new SheetWidget(0, centerAreaWidget);
+				centerAreaWidget->addWidget(sheetWidget);
+				m_sheetWidgets.insert(0, sheetWidget);
 			}
 		}
+		m_snapAction->setEnabled(false);
+		m_effectAction->setEnabled(false);
+		m_followAction->setEnabled(false);
 	} else {
-		songWidget = m_songWidgets.value(song);
+		sheetWidget = m_sheetWidgets.value(sheet);
+		connect(sheet, SIGNAL(snapChanged()), this, SLOT(update_snap_state()));
+		connect(sheet, SIGNAL(modeChanged()), this, SLOT(update_effects_state()));
+		connect(sheet, SIGNAL(tempFollowChanged(bool)), this, SLOT(update_temp_follow_state(bool)));
+		connect(sheet, SIGNAL(transportStopped()), this, SLOT(update_follow_state()));
+		update_snap_state();
+		update_effects_state();
+		m_snapAction->setEnabled(true);
+		m_effectAction->setEnabled(true);
+		m_followAction->setEnabled(true);
 	}
 	
-	if (!songWidget) {
-		songWidget = new SongWidget(song, centerAreaWidget);
-		centerAreaWidget->addWidget(songWidget);
-		m_songWidgets.insert(song, songWidget);
+	if (!sheetWidget) {
+		sheetWidget = new SheetWidget(sheet, centerAreaWidget);
+		centerAreaWidget->addWidget(sheetWidget);
+		m_sheetWidgets.insert(sheet, sheetWidget);
 	}
-	currentSongWidget = songWidget;
-	centerAreaWidget->setCurrentIndex(centerAreaWidget->indexOf(songWidget));
-	songWidget->setFocus();
+	currentSheetWidget = sheetWidget;
+	centerAreaWidget->setCurrentIndex(centerAreaWidget->indexOf(sheetWidget));
+	sheetWidget->setFocus();
 	
-	if (song) {
-		pm().get_undogroup()->setActiveStack(song->get_history_stack());
+	if (sheet) {
+		pm().get_undogroup()->setActiveStack(sheet->get_history_stack());
 	}
 }
 
@@ -446,112 +495,176 @@ Command * Interface::show_cd_writing_dialog( )
 
 void Interface::create_menus( )
 {
-	QMenu* menu;
 	QAction* action;
-	 
-	menu = menuBar()->addMenu(tr("&Project"));
+
+	m_projectMenu = menuBar()->addMenu(tr("&Project"));
 	
-	action = menu->addAction(tr("&New..."));
-	action->setIcon(find_pixmap(":/new-16"));
+	action = m_projectMenu->addAction(tr("&New..."));
+	action->setIcon(find_pixmap(":/new"));
 	action->setShortcuts(QKeySequence::New);
+	m_projectToolBar->addAction(action);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_newproject_dialog()));
 	
-	action = menu->addAction(tr("&Open..."));
-	action->setIcon(style()->standardIcon(QStyle::SP_FileDialogContentsView));
+	action = m_projectMenu->addAction(tr("&Open..."));
+	action->setIcon(QIcon(":/open"));
 	action->setShortcuts(QKeySequence::Open);
+	m_projectToolBar->addAction(action);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_open_project_dialog()));
 	
-	menu->addSeparator();
-	
-	action = menu->addAction(tr("&Save"));
-	action->setShortcuts(QKeySequence::Save);
+	action = m_projectMenu->addAction(tr("&Save"));
 	m_projectSaveAction = action;
-	action->setIcon(style()->standardIcon(QStyle::SP_DialogSaveButton));
+	action->setShortcuts(QKeySequence::Save);
+	action->setIcon(QIcon(":/save"));
+	m_projectToolBar->addAction(action);
 	connect(action, SIGNAL(triggered(bool)), &pm(), SLOT(save_project()));
-	
-	action = menu->addAction(tr("&Manage Project..."));
+
+	m_projectMenu->addSeparator();
+
+	action = m_projectMenu->addAction(tr("&Manage Project..."));
+	m_projectSheetManagerAction = action;
 	QList<QKeySequence> list;
 	list.append(QKeySequence("F4"));
-	action->setShortcuts(list);
-	action->setIcon(QIcon(find_pixmap(":/songmanager-16")));
-	m_projectSongManagerAction = action;
+	m_projectSheetManagerAction->setShortcuts(list);
+	m_projectSheetManagerAction->setIcon(QIcon(":/projectmanager"));
+	m_projectToolBar->addAction(m_projectSheetManagerAction);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_project_manager_dialog()));
 	
-	action = menu->addAction(tr("&Export..."));
+	action = m_projectMenu->addAction(tr("&Export..."));
+	m_projectExportAction = action;
 	list.clear();
 	list.append(QKeySequence("F9"));
-	action->setShortcuts(list);
-	action->setIcon(style()->standardIcon(QStyle::SP_DialogApplyButton));
-	m_projectExportAction = action;
+	m_projectExportAction->setShortcuts(list);
+	m_projectExportAction->setIcon(QIcon(":/export"));
+	m_projectToolBar->addAction(m_projectExportAction);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_export_widget()));
 	
-	action = menu->addAction(tr("&CD Writing..."));
+	action = m_projectMenu->addAction(tr("&CD Writing..."));
 	list.clear();
 	list.append(QKeySequence("F8"));
 	action->setShortcuts(list);
-	action->setIcon(style()->standardIcon(QStyle::SP_DialogApplyButton));
+	action->setIcon(QIcon(":/write-cd"));
+	m_projectToolBar->addAction(action);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_cd_writing_dialog()));
 	
-	action = menu->addAction(tr("&Restore Backup..."));
+	action = m_projectMenu->addAction(tr("&Restore Backup..."));
 	list.clear();
 	list.append(QKeySequence("F10"));
 	action->setShortcuts(list);
-	action->setIcon(style()->standardIcon(QStyle::SP_FileDialogBack));
+	action->setIcon(QIcon(":/restore"));
+	m_projectToolBar->addAction(action);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_restore_project_backup_dialog()));
 	
-	menu->addSeparator();
+	m_projectMenu->addSeparator();
 	
-	action = menu->addAction(tr("&Quit"));
+	action = m_projectMenu->addAction(tr("&Quit"));
 	list.clear();
+	// FIXME using CTRL + Q gives a warning about deleting an object in it's event handler!!!!!!
 	list.append(QKeySequence("CTRL+Q"));
 	action->setShortcuts(list);
-	action->setIcon(QIcon(find_pixmap(":/exit-16")));
+	action->setIcon(QIcon(":/exit"));
 	connect(action, SIGNAL(triggered( bool )), &pm(), SLOT(exit()));
 	
-	
-	menu = menuBar()->addMenu(tr("&Sheet"));
-	m_songMenuAction = menu->menuAction();
-	
-	action = menu->addAction(tr("New &Track(s)..."));
-	connect(action, SIGNAL(triggered()), this, SLOT(show_newtrack_dialog()));
-	action = menu->addAction(tr("New &Sheet(s)..."));
-	connect(action, SIGNAL(triggered()), this, SLOT(show_newsong_dialog()));
+	m_editMenu = menuBar()->addMenu(tr("&Edit"));
 
-	menu->addSeparator();
-	
-	action = menu->addAction(tr("Marker Editor..."));
-	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_marker_dialog()));
-	
-	menu->addSeparator();
+	action = m_editMenu->addAction(tr("Undo"));
+	action->setIcon(QIcon(":/undo"));
+	action->setShortcuts(QKeySequence::Undo);
+	m_editToolBar->addAction(action);
+	connect(action, SIGNAL(triggered( bool )), &pm(), SLOT(undo()));
 
-	action = menu->addAction(tr("Import &Audio..."));
+	action = m_editMenu->addAction(tr("Redo"));
+	action->setIcon(QIcon(":/redo"));
+	action->setShortcuts(QKeySequence::Redo);
+	m_editToolBar->addAction(action);
+	connect(action, SIGNAL(triggered( bool )), &pm(), SLOT(redo()));	
+
+	m_editMenu->addSeparator();
+	m_editToolBar->addSeparator();
+
+	action = m_editMenu->addAction(tr("Import &Audio..."));
+	action->setIcon(QIcon(":/import-audio"));
+	m_editToolBar->addAction(action);
 	connect(action, SIGNAL(triggered()), this, SLOT(import_audio()));
-	action = menu->addAction(tr("Insert Si&lence..."));
-	connect(action, SIGNAL(triggered()), this, SLOT(show_insertsilence_dialog()));
-	
-	
-	menu = menuBar()->addMenu(tr("&View"));
 
-	menu->addAction(historyDW->toggleViewAction());
-	menu->addAction(busMonitorDW->toggleViewAction());
-	menu->addAction(AudioSourcesDW->toggleViewAction());
+	action = m_editMenu->addAction(tr("Insert Si&lence..."));
+	action->setIcon(QIcon(":/import-silence"));
+	m_editToolBar->addAction(action);
+	connect(action, SIGNAL(triggered()), this, SLOT(show_insertsilence_dialog()));
+
+	m_editMenu->addSeparator();
+	m_editToolBar->addSeparator();
+
+	m_snapAction = m_editMenu->addAction(tr("&Snap"));
+	m_snapAction->setIcon(QIcon(":/snap"));
+	m_snapAction->setCheckable(true);
+	m_snapAction->setToolTip(tr("Snap items to edges of other items while dragging."));
+	m_editToolBar->addAction(m_snapAction);
+	connect(m_snapAction, SIGNAL(triggered(bool)), this, SLOT(snap_state_changed(bool)));
+
+	m_followAction = m_editMenu->addAction(tr("S&croll Playback"));
+	m_followAction->setIcon(QIcon(":/follow"));
+	m_followAction->setCheckable(true);
+	m_followAction->setToolTip(tr("Keep play cursor in view while playing or recording."));
+	m_editToolBar->addAction(m_followAction);
+	connect(m_followAction, SIGNAL(triggered(bool)), this, SLOT(follow_state_changed(bool)));
+
+	m_effectAction = m_editMenu->addAction(tr("&Show Effects"));
+	m_effectAction->setIcon(QIcon(":/effects"));
+	m_effectAction->setCheckable(true);
+	m_effectAction->setToolTip(tr("Show effect plugins and automation curves on tracks"));
+	m_editToolBar->addAction(m_effectAction);
+	connect(m_effectAction, SIGNAL(triggered(bool)), this, SLOT(effect_state_changed(bool)));
+
+	m_viewMenu = menuBar()->addMenu(tr("&View"));
+
+	m_viewMenu->addAction(historyDW->toggleViewAction());
+	m_viewMenu->addAction(busMonitorDW->toggleViewAction());
+	m_viewMenu->addAction(AudioSourcesDW->toggleViewAction());
+
+	action = m_viewMenu->addAction(tr("Marker Editor..."));
+	connect(action, SIGNAL(triggered(bool)), this, SLOT(show_marker_dialog()));
+
+	m_viewMenu->addSeparator();
 	
-	menu->addSeparator();
+	m_viewMenu->addAction(correlationMeterDW->toggleViewAction());
+	m_viewMenu->addAction(spectralMeterDW->toggleViewAction());
+
+	m_viewMenu->addSeparator();
+	QAction* toolbars = m_viewMenu->addAction("ToolBars");
+	toolbars->setEnabled(false);
+	m_viewMenu->addSeparator();
 	
-	menu->addAction(correlationMeterDW->toggleViewAction());
-	menu->addAction(spectralMeterDW->toggleViewAction());
-	
-	menu->addSeparator();
-	
-	menu->addAction(m_infoBar->toggleViewAction());
-	m_infoBar->toggleViewAction()->setText(tr("Sheet Toolbar"));
-	menu->addAction(m_sysinfo->toggleViewAction());
+	m_viewMenu->addAction(transportConsole->toggleViewAction());
+	transportConsole->toggleViewAction()->setText(tr("Transport Console"));
+
+	// if unifiedTitleAndToolBarOnMac == true we don't want the main toolbars
+	// to be hidden. thus only add the menu entries on systems != OS X
+#if !defined (Q_WS_MAC)
+	m_viewMenu->addAction(m_projectToolBar->toggleViewAction());
+	m_projectToolBar->toggleViewAction()->setText(tr("Project"));
+
+	m_viewMenu->addAction(m_editToolBar->toggleViewAction());
+	m_editToolBar->toggleViewAction()->setText(tr("Edit"));
+#endif
+
+	m_viewMenu->addAction(m_sysinfo->toggleViewAction());
 	m_sysinfo->toggleViewAction()->setText(tr("System Information"));
+
+	m_sheetMenu = menuBar()->addMenu(tr("&Sheet"));
+	m_sheetMenuAction = m_sheetMenu->menuAction();
+
+	action = m_sheetMenu->addAction(tr("New &Sheet(s)..."));
+	action->setIcon(QIcon(":/new-sheet"));
+	connect(action, SIGNAL(triggered()), this, SLOT(show_newsheet_dialog()));
+		
+	action = m_sheetMenu->addAction(tr("New &Track(s)..."));
+	connect(action, SIGNAL(triggered()), this, SLOT(show_newtrack_dialog()));
+
+	m_sheetMenu->addSeparator();
 	
+	m_settingsMenu = menuBar()->addMenu(tr("Se&ttings"));
 	
-	menu = menuBar()->addMenu(tr("Se&ttings"));
-	
-	m_encodingMenu = menu->addMenu(tr("&Recording File Format"));
+	m_encodingMenu = m_settingsMenu->addMenu(tr("&Recording File Format"));
 	
 	action = m_encodingMenu->addAction("WAVE");
 	action->setData("wav");
@@ -563,43 +676,38 @@ void Interface::create_menus( )
 	action->setData("w64");
 	connect(action, SIGNAL(triggered( bool )), this, SLOT(change_recording_format_to_wav64()));
 	
-	m_resampleQualityMenu = menu->addMenu(tr("&Resample Quality"));
-	action = m_resampleQualityMenu->addAction("Best");
+	m_resampleQualityMenu = m_settingsMenu->addMenu(tr("&Resample Quality"));
+	action = m_resampleQualityMenu->addAction(tr("Best"));
 	action->setData(0);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(change_resample_quality_to_best()));
-	action = m_resampleQualityMenu->addAction("High");
+	action = m_resampleQualityMenu->addAction(tr("High"));
 	action->setData(1);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(change_resample_quality_to_high()));
-	action = m_resampleQualityMenu->addAction("Medium");
+	action = m_resampleQualityMenu->addAction(tr("Medium"));
 	action->setData(2);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(change_resample_quality_to_medium()));
-	action = m_resampleQualityMenu->addAction("Fast");
+	action = m_resampleQualityMenu->addAction(tr("Fast"));
 	action->setData(3);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(change_resample_quality_to_fast()));
 
 	// fake a config changed 'signal-slot' action, to set the encoding menu icons
 	config_changed();
 	
-	menu->addSeparator();
+	m_settingsMenu->addSeparator();
 	
-	menu->addAction(m_infoBar->get_snap_action());
-	menu->addAction(m_infoBar->get_follow_action());
-	
-	menu->addSeparator();
-	
-	action = menu->addAction(tr("&Preferences..."));
+	action = m_settingsMenu->addAction(tr("&Preferences..."));
 	connect(action, SIGNAL(triggered( bool )), this, SLOT(show_settings_dialog()));
 	
 	
-	menu = menuBar()->addMenu("&Help");
-	action = menu->addAction(tr("&Getting Started"));
+	m_helpMenu = menuBar()->addMenu(tr("&Help"));
+	action = m_helpMenu->addAction(tr("&Getting Started"));
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(quick_start()));
 	
-	action = menu->addAction(tr("&User Manual"));
+	action = m_helpMenu->addAction(tr("&User Manual"));
 	action->setIcon(style()->standardIcon(QStyle::SP_DialogHelpButton));
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(open_help_browser()));
 	
-	action = menu->addAction(tr("&About Traverso"));
+	action = m_helpMenu->addAction(tr("&About Traverso"));
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(about_traverso()));
 }
 
@@ -655,7 +763,7 @@ Command * Interface::show_context_menu( )
 			toplevelmenu = m_contextMenus.value(className);
 		
 			if ( ! toplevelmenu ) {
-				printf("No menu for %s, creating new one\n", QS_C(className));
+				printf("Interface: No menu for %s, creating new one\n", QS_C(className));
 				toplevelmenu = create_context_menu(item);
 				if (! toplevelmenu ) {
 					if (items.size() > 1) {
@@ -680,12 +788,10 @@ Command * Interface::show_context_menu( )
 			action = toplevelmenu->insertMenu(action, menu);
 			QString name = className.remove("View");
 
-			if (name == "Song") name = "Sheet"; // FIXME!!!
-
-			action->setText(name);
+			action->setText(QObject::tr(QS_C(name)));
 		}
 	}
-	
+
 	// It's impossible there is NO toplevelmenu, but oh well...
 	if (toplevelmenu) {
 		toplevelmenu->exec(QCursor::pos());
@@ -738,7 +844,7 @@ Command * Interface::get_keymap(QString &str)
 	
 	QMap<QString, QList<const QMetaObject*> > objects;
 	
-	QList<const QMetaObject*> songlist; songlist << &Song::staticMetaObject; songlist << &SongView::staticMetaObject;
+	QList<const QMetaObject*> sheetlist; sheetlist << &Sheet::staticMetaObject; sheetlist << &SheetView::staticMetaObject;
 	QList<const QMetaObject*> tracklist; tracklist << &Track::staticMetaObject; tracklist << &TrackView::staticMetaObject;
 	QList<const QMetaObject*> cliplist; cliplist << &AudioClip::staticMetaObject; cliplist << &AudioClipView::staticMetaObject;
 	QList<const QMetaObject*> curvelist; curvelist << &Curve::staticMetaObject; curvelist << &CurveView::staticMetaObject;
@@ -749,7 +855,7 @@ Command * Interface::get_keymap(QString &str)
 	QList<const QMetaObject*> interfacelist; interfacelist << &Interface::staticMetaObject;
 	QList<const QMetaObject*> pmlist; pmlist << &ProjectManager::staticMetaObject;
 		
-	objects.insert("Sheet", songlist);
+	objects.insert("Sheet", sheetlist);
 	objects.insert("Track", tracklist);
 	objects.insert("AudioClip", cliplist);
 	objects.insert("Curve", curvelist);
@@ -838,12 +944,10 @@ QMenu* Interface::create_context_menu(QObject* item, QList<MenuData >* menulist)
 		name = "noname";
 	}
 
-	if (name == "Song") name = "Sheet"; // FIXME!!!
-
 	QMenu* menu = new QMenu(this);
 	menu->installEventFilter(this);
 	
-	QAction* menuAction = menu->addAction(name);
+	QAction* menuAction = menu->addAction(QObject::tr(QS_C(name)));
 	QFont font(themer()->get_font("ContextMenu:fontscale:actions"));
 	font.setBold(true);
 	menuAction->setFont(font);
@@ -851,7 +955,7 @@ QMenu* Interface::create_context_menu(QObject* item, QList<MenuData >* menulist)
 	menu->addSeparator();
 	menu->setFont(themer()->get_font("ContextMenu:fontscale:actions"));
 	
-	QHash<QString, QList<MenuData>* > submenus;
+	QMap<QString, QList<MenuData>* > submenus;
 	
 	for (int i=0; i<list.size(); ++i) {
 		MenuData data = list.at(i);
@@ -893,7 +997,7 @@ QMenu* Interface::create_context_menu(QObject* item, QList<MenuData >* menulist)
 	// actions, a little code duplication here, adding action to the 
 	// menu is also done ~10 lines up ...
 	QList<QString> keys = submenus.keys();
-	foreach(QString key, keys) {
+	foreach(const QString &key, keys) {
 		QList<MenuData>* list = submenus.value(key);
 		
 		qSort(list->begin(), list->end(), MenuData::smaller);
@@ -906,7 +1010,7 @@ QMenu* Interface::create_context_menu(QObject* item, QList<MenuData >* menulist)
 		sub->menuAction()->setFont(font);
 		
 		QAction* action = menu->insertMenu(0, sub);
-		action->setText(key);
+		action->setText(QObject::tr(QS_C(key)));
 		foreach(MenuData data, *list) {
 			QAction* action = new QAction(sub);
 			QString keyfact = create_keyfact_string(data.keysequence, data.modifierkeys);
@@ -1014,13 +1118,13 @@ QMenu* Interface::create_fade_selector_menu(const QString& fadeTypeName)
 
 void Interface::config_changed()
 {
-	bool toggled = config().get_property("Interface", "OpenGL", false).toBool();
+/*	bool toggled = config().get_property("Interface", "OpenGL", false).toBool();
 
-	foreach(SongWidget* widget, m_songWidgets) {
+	foreach(SheetWidget* widget, m_sheetWidgets) {
 		widget->set_use_opengl(toggled);
-	}
+	}*/
 	
-	QString encoding = config().get_property("Recording", "FileFormat", "").toString();
+	QString encoding = config().get_property("Recording", "FileFormat", "wav").toString();
 	QList<QAction* > actions = m_encodingMenu->actions();
 	
 	foreach(QAction* action, actions) {
@@ -1031,9 +1135,17 @@ void Interface::config_changed()
 		}
 	}
 	
-	int quality = config().get_property("Conversion", "RTResamplingConverterType", 2).toInt();
+	int quality = config().get_property("Conversion", "RTResamplingConverterType", DEFAULT_RESAMPLE_QUALITY).toInt();
 	actions = m_resampleQualityMenu->actions();
 	
+	bool useResampling = config().get_property("Conversion", "DynamicResampling", true).toBool();
+	if (useResampling) {
+		m_resampleQualityMenu->setEnabled(true);
+	} else {
+		m_resampleQualityMenu->setEnabled(false);
+	}
+
+	
 	foreach(QAction* action, actions) {
 		if (action->data().toInt() == quality) {
 			action->setIcon(style()->standardIcon(QStyle::SP_DialogApplyButton));
@@ -1041,23 +1153,105 @@ void Interface::config_changed()
 			action->setIcon(QIcon());
 		}
 	}
+
+	switch (config().get_property("Themer", "toolbuttonstyle", 0).toInt()) {
+		case 0:
+			m_projectToolBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
+			m_editToolBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
+			break;
+
+		case 1:
+			m_projectToolBar->setToolButtonStyle(Qt::ToolButtonTextOnly);
+			m_editToolBar->setToolButtonStyle(Qt::ToolButtonTextOnly);
+			break;
+
+		case 2:
+			m_projectToolBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+			m_editToolBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+			break;
+
+		case 3:
+			m_projectToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+			m_editToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+			break;
+	}
+
+	int iconsize = config().get_property("Themer", "iconsize", "22").toInt();
+	m_projectToolBar->setIconSize(QSize(iconsize, iconsize));
+	m_editToolBar->setIconSize(QSize(iconsize, iconsize));
+
+	int transportconsolesize = config().get_property("Themer", "transportconsolesize", "22").toInt();
+	transportConsole->setIconSize(QSize(transportconsolesize, transportconsolesize));
+	transportConsole->resize(transportConsole->sizeHint());
 }
 
 void Interface::import_audio()
 {
-	if (currentSongWidget->get_song()->get_numtracks() > 0) {
-		APILinkedList tracks = currentSongWidget->get_song()->get_tracks();
-		Track*	shortestTrack = (Track*)tracks.first();
+	if (!currentSheetWidget->get_sheet()->get_numtracks()) {
+		return;
+	}
 
-		apill_foreach(Track* track, Track, tracks) {
-			if (track->get_cliplist().last() && ((AudioClip*)track->get_cliplist().last())->get_track_end_location() > ((AudioClip*)shortestTrack->get_cliplist().last())->get_track_end_location()) {
-				shortestTrack = track;
-			}
+	QStringList files = QFileDialog::getOpenFileNames(this, tr("Open Audio Files"),
+			config().get_property("Project", "directory", "/directory/unknown").toString(),
+			tr("Audio files (*.wav *.flac *.ogg *.mp3 *.wv *.w64)"));
+
+	QList<Track*> tracks = currentSheetWidget->get_sheet()->get_tracks();
+	Track*	track = tracks.first();
+	bool markers = false;
+
+	ImportClipsDialog *importClips = new ImportClipsDialog(this);
+
+	importClips->set_tracks(tracks);
+
+	if (importClips->exec() == QDialog::Accepted) {
+		if (!importClips->has_tracks()) {
+			delete importClips;
+			return;
 		}
 
-		Import* cmd = new Import(shortestTrack, TimeRef());
-		Command::process_command(cmd);
+		track = importClips->get_selected_track();
+		markers = importClips->get_add_markers();
 	}
+
+	// append the clips to the selected track
+	TimeRef position = TimeRef();
+	if (!track->get_cliplist().isEmpty()) {
+		position = (track->get_cliplist().last())->get_track_end_location();
+	}
+
+	TimeLine* tl = currentSheetWidget->get_sheet()->get_timeline();
+	int n = tl->get_markers().size() + 1;
+	if (tl->has_end_marker()) {
+		n -= 1;
+	}
+
+	while(!files.isEmpty()) {
+		QString file = files.takeFirst();
+		Import* import = new Import(file);
+		import->set_track(track);
+		import->set_position(position);
+
+		QFileInfo fi(file);
+		Marker* m = new Marker(tl, position);
+		m->set_description(QString(tr("%1: %2")).arg(n).arg(fi.baseName()));
+
+		if (import->create_readsource() != -1) {
+			position += import->readsource()->get_length();
+			Command::process_command(import);
+			Command::process_command(tl->add_marker(m, true));
+		}
+		++n;
+	}
+
+	if (tl->has_end_marker()) {
+		Marker* m = tl->get_end_marker();
+		m->set_when(position);
+	} else {
+		Marker* m = new Marker(tl, position, Marker::ENDMARKER);
+		Command::process_command(tl->add_marker(m, true));
+	}
+
+	delete importClips;
 }
 
 DigitalClock::DigitalClock(QWidget *parent)
@@ -1095,6 +1289,12 @@ void Interface::show_settings_dialog()
 	m_settingsdialog->show();
 }
 
+void Interface::show_settings_dialog_sound_system_page()
+{
+	show_settings_dialog();
+	m_settingsdialog->show_page("Sound System");
+}
+
 
 void Interface::closeEvent(QCloseEvent * event)
 {
@@ -1124,6 +1324,10 @@ Command * Interface::show_newproject_dialog()
 {
 	if (! m_newProjectDialog ) {
 		m_newProjectDialog = new NewProjectDialog(this);
+		AudioFileCopyConvert* m_converter = m_newProjectDialog->get_converter();
+		connect(m_converter, SIGNAL(taskStarted(QString)), m_progressBar, SLOT(set_label(QString)));
+		connect(m_converter, SIGNAL(progress(int)), m_progressBar, SLOT(set_progress(int)));
+		connect(m_newProjectDialog, SIGNAL(number_of_files(int)), m_progressBar, SLOT(set_num_files(int)));
 	}
 	m_newProjectDialog->show();
 	return 0;
@@ -1145,12 +1349,11 @@ Command * Interface::show_insertsilence_dialog()
 
 Command * Interface::show_marker_dialog()
 {
-	if (! m_markerDialog ) {
-		m_markerDialog = new MarkerDialog(this);
-	}
-	m_markerDialog->song_to_be_showed(pm().get_project()->get_current_song());
-	m_markerDialog->show();
-	
+	MarkerDialog* markerDialog = new MarkerDialog(this);
+
+	markerDialog->exec();
+	delete markerDialog;
+
 	return 0;
 }
 
@@ -1159,13 +1362,13 @@ QSize Interface::sizeHint() const
 	return QSize(800, 600);
 }
 
-Command* Interface::show_newsong_dialog()
+Command* Interface::show_newsheet_dialog()
 {
-	if (! m_newSongDialog) {
-		m_newSongDialog = new NewSongDialog(this);
+	if (! m_newSheetDialog) {
+		m_newSheetDialog = new NewSheetDialog(this);
 	}
 	
-	m_newSongDialog->show();
+	m_newSheetDialog->show();
 	
 	return 0;
 }
@@ -1290,3 +1493,192 @@ void Interface::save_config_and_emit_message(const QString & message)
 	config().save();
 }
 
+
+
+Command * Interface::start_transport()
+{
+	Project* project = pm().get_project();
+	if (project) {
+		Sheet* sheet = project->get_current_sheet();
+		if (sheet) {
+			return sheet->start_transport();
+		}
+	}
+	
+	return 0;
+}
+
+Command * Interface::set_recordable_and_start_transport()
+{
+	if (m_project) {
+		Sheet* sheet = m_project->get_current_sheet();
+		if (sheet) {
+			return sheet->set_recordable_and_start_transport();
+		}
+	}
+	
+	return 0;
+}
+
+// snapping is a global property and should be stored in each sheet
+void Interface::snap_state_changed(bool state)
+{
+	if (m_project) {
+		QList<Sheet* > sheetlist = m_project->get_sheets();
+		foreach( Sheet* sheet, sheetlist) {
+			sheet->set_snapping(state);
+		}
+	}
+}
+
+void Interface::update_snap_state()
+{
+	if (m_project) {
+		bool snapping = m_project->get_current_sheet()->is_snap_on();
+		m_snapAction->setChecked(snapping);
+	}
+}
+
+// scrolling is a global property but should not be stored in the sheets
+void Interface::update_follow_state()
+{
+	m_isFollowing = config().get_property("PlayHead", "Follow", true).toBool();
+	m_followAction->setChecked(m_isFollowing);
+}
+
+void Interface::update_temp_follow_state(bool state)
+{
+	if (m_project->get_current_sheet()->is_transport_rolling() && m_isFollowing) {
+		m_followAction->setChecked(state);
+	}
+}
+
+void Interface::follow_state_changed(bool state)
+{
+	Sheet* sheet = m_project->get_current_sheet();
+
+	if (!sheet) {
+		return;
+	}
+	
+	if (!sheet->is_transport_rolling() || !m_isFollowing) {
+		m_isFollowing = state;
+		config().set_property("PlayHead", "Follow", state);
+		config().save();
+		if (sheet->is_transport_rolling()) {
+			sheet->set_temp_follow_state(state);
+		}
+	} else {
+		sheet->set_temp_follow_state(state);
+	}
+}
+
+// the view mode is a sheet property and should be stored in the sheet
+void Interface::effect_state_changed(bool state)
+{
+	Sheet* sheet = m_project->get_current_sheet();
+
+	if (state) {
+		sheet->set_effects_mode();
+	} else {
+		sheet->set_editing_mode();
+	}
+}
+
+void Interface::update_effects_state()
+{
+	Sheet* sheet = m_project->get_current_sheet();
+
+	if (!sheet) {
+		return;
+	}
+	
+	if (sheet->get_mode() == Sheet::EDIT) {
+		m_effectAction->setChecked(false);
+	} else {
+		m_effectAction->setChecked(true);
+	}
+}
+
+void Interface::sheet_selector_update_sheets()
+{
+	// empty the list, make sure everything is deleted
+	while(!m_currentSheetActions.isEmpty())
+	{
+		QAction *action = m_currentSheetActions.takeFirst();
+		delete action;
+	}
+
+	if (!m_project)
+	{
+		return;
+	}
+
+	if (!m_project->get_current_sheet())
+	{
+		return;
+	}
+
+	qint64 id = m_project->get_current_sheet()->get_id();
+
+	QActionGroup* actiongroup = new QActionGroup(this);
+	actiongroup->setExclusive(true);
+
+	// create the new actions
+	foreach(Sheet* sheet, m_project->get_sheets())
+	{
+		QString string = QString::number(m_project->get_sheet_index(sheet->get_id())) +
+		": " + sheet->get_title();
+		QAction* action = m_sheetMenu->addAction(string);
+		actiongroup->addAction(action);
+		action->setData(sheet->get_id());
+		action->setCheckable(true);
+
+		if (sheet->get_id() == id)
+		{
+			action->setChecked(true);
+		} else {
+			action->setChecked(false);
+		}
+
+		connect(action, SIGNAL(triggered()), this, SLOT(sheet_selected()));
+		m_currentSheetActions.append(action);
+	}
+}
+
+void Interface::sheet_selected()
+{
+	// identify the action that was activated
+	QAction *orig = qobject_cast<QAction *>(sender());
+
+	if (!orig)
+	{
+		return;
+	}
+
+	qint64 id = orig->data().toLongLong();
+
+	// uncheck all other actions
+	foreach(QAction* action, m_currentSheetActions)
+	{
+		if (action->data().toLongLong() != id)
+		{
+			action->setChecked(false);
+		}
+	}
+
+	m_project->set_current_sheet(id);
+}
+
+void Interface::sheet_selector_sheet_added(Sheet* sheet)
+{
+	connect(sheet, SIGNAL(propertyChanged()), this, SLOT(sheet_selector_update_sheets()));
+	sheet_selector_update_sheets();
+}
+
+void Interface::sheet_selector_sheet_removed(Sheet* sheet)
+{
+	disconnect(sheet, SIGNAL(propertyChanged()), this, SLOT(sheet_selector_update_sheets()));
+	sheet_selector_update_sheets();
+}
+
