@@ -26,7 +26,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include <QMainWindow>
 #include <QHash>
 
-class Song;
+class Sheet;
 class Track;
 class Project;
 class BusMonitor;
@@ -47,11 +47,10 @@ class QToolBar;
 class QToolButton;
 class ResourcesWidget;
 class ResourcesInfoWidget;
-class DriverInfoWidget;
-class HDDSpaceInfoWidget;
-class SongWidget;
+class SheetWidget;
 class CorrelationMeterWidget;
 class SpectralMeterWidget;
+class TransportConsoleWidget;
 class SettingsDialog;
 class ProjectManagerDialog;
 class OpenProjectDialog;
@@ -60,11 +59,12 @@ class SysInfoToolBar;
 class InsertSilenceDialog;
 class MarkerDialog;
 class BusSelectorDialog;
-class NewSongDialog;
+class NewSheetDialog;
 class NewTrackDialog;
 class NewProjectDialog;
 class Ui_QuickStartDialog;
 class RestoreProjectBackupDialog;
+class ProgressToolBar;
 struct MenuData;
 
 class Interface : public QMainWindow
@@ -76,6 +76,9 @@ class Interface : public QMainWindow
 	Q_CLASSINFO("show_project_manager_dialog", tr("Show Project Management Dialog"))
 	Q_CLASSINFO("full_screen", tr("Full Screen"))
 	Q_CLASSINFO("export_keymap", tr("Export keymap"))
+	Q_CLASSINFO("start_transport", tr("Play"))
+	Q_CLASSINFO("set_recordable_and_start_transport", tr("Record"));
+
 
 public :
 	Interface();
@@ -98,8 +101,8 @@ protected:
 
 private:
 	QStackedWidget* 	centerAreaWidget;
-	QHash<Song*, SongWidget* > m_songWidgets;
-	SongWidget*		currentSongWidget;
+	QHash<Sheet*, SheetWidget* > m_sheetWidgets;
+	SheetWidget*		currentSheetWidget;
 	QList<ViewPort* > 	currentProjectViewPortList;
 	QHash<QString, QMenu*>	m_contextMenus;
 	ExportDialog*		m_exportDialog;
@@ -111,36 +114,45 @@ private:
 	ResourcesWidget* 	audiosourcesview;
 	QDockWidget*		correlationMeterDW;
 	CorrelationMeterWidget*	correlationMeter;
+	TransportConsoleWidget*	transportConsole;
 	QDockWidget*		spectralMeterDW;
 	SpectralMeterWidget*	spectralMeter;
 	SettingsDialog*		m_settingsdialog;
 	ProjectManagerDialog*	m_projectManagerDialog;
 	OpenProjectDialog*	m_openProjectDialog;
 	InsertSilenceDialog*	m_insertSilenceDialog;
-	MarkerDialog*		m_markerDialog;
-	InfoToolBar* 		m_infoBar;
 	SysInfoToolBar* 	m_sysinfo;
+	ProgressToolBar*	m_progressBar;
 	BusSelectorDialog*	m_busSelector;
-	NewSongDialog*		m_newSongDialog;
+	NewSheetDialog*		m_newSheetDialog;
 	NewTrackDialog*		m_newTrackDialog;
 	NewProjectDialog*	m_newProjectDialog;
 	QDialog*		m_quickStart;
 	RestoreProjectBackupDialog* m_restoreProjectBackupDialog;
-
+	Project*		m_project;
+	bool			m_isFollowing;
 
 	BusMonitor* 		busMonitor;
 	QToolBar* 		mainToolBar;
+	QToolBar*		m_projectToolBar;
+	QToolBar*		m_editToolBar;
 	QToolButton*		openGlButton;
 	QAction*		m_projectSaveAction;
-	QAction*		m_projectSongManagerAction;
+	QAction*		m_projectSheetManagerAction;
 	QAction*		m_projectExportAction;
-	QAction*		m_songMenuAction;
+	QAction*		m_sheetMenuAction;
+	QAction*		m_snapAction;
+	QAction*		m_followAction;
+	QAction*		m_effectAction;
 	QMenu*			m_encodingMenu;
 	QMenu*			m_resampleQualityMenu;
-	
-	ResourcesInfoWidget*	resourcesInfo;
-	DriverInfoWidget*	driverInfo;
-	HDDSpaceInfoWidget*	hddInfo;
+	QMenu*			m_projectMenu;
+	QMenu*			m_sheetMenu;
+	QMenu*			m_editMenu;
+	QMenu*			m_viewMenu;
+	QMenu*			m_settingsMenu;
+	QMenu*			m_helpMenu;
+	QList<QAction*>		m_currentSheetActions;
 	
 	void create_menus();
 	void save_config_and_emit_message(const QString& message);
@@ -152,8 +164,9 @@ private:
 
 public slots :
 	void set_project(Project* project);
-	void show_song(Song* song);
+	void show_sheet(Sheet* sheet);
 	void show_settings_dialog();
+	void show_settings_dialog_sound_system_page();
 	void open_help_browser();
 	void process_context_menu_action(QAction* action);
 	void set_fade_in_shape(QAction* action);
@@ -182,15 +195,29 @@ public slots :
 	Command* show_restore_project_backup_dialog(QString projectdir);
 	Command* show_insertsilence_dialog();
 	Command* show_marker_dialog();
-	Command* show_newsong_dialog();
+	Command* show_newsheet_dialog();
 	Command* show_newtrack_dialog();
 	Command* show_newproject_dialog();
+	Command* start_transport();
+	Command* set_recordable_and_start_transport();
+
 	
 private slots:
-	void delete_songwidget(Song*);
+	void delete_sheetwidget(Sheet*);
 	void project_dir_change_detected();
 	void project_load_failed(QString project, QString reason);
 	void project_file_mismatch(QString rootdir, QString projectname);
+	void snap_state_changed(bool state);
+	void update_snap_state();
+	void effect_state_changed(bool state);
+	void update_effects_state();
+	void follow_state_changed(bool state);
+	void update_follow_state();
+	void update_temp_follow_state(bool state);
+	void sheet_selector_update_sheets();
+	void sheet_selected();
+	void sheet_selector_sheet_added(Sheet*);
+	void sheet_selector_sheet_removed(Sheet*);
 };
 
 
