@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2007 Remon Sijrier 
+    Copyright (C) 2007-2008 Remon Sijrier 
  
     This file is part of Traverso
  
@@ -25,6 +25,12 @@
 #include "ui_NewProjectDialog.h"
 #include <QDialog>
 
+class AudioFileCopyConvert;
+class ExportFormatOptionsWidget;
+class QButtonGroup;
+struct ExportSpecification;
+
+
 class NewProjectDialog : public QDialog, protected Ui::NewProjectDialog
 {
 	Q_OBJECT
@@ -33,10 +39,30 @@ public:
 	NewProjectDialog(QWidget* parent = 0);
 	~NewProjectDialog();
 
+	AudioFileCopyConvert* get_converter();
+
+private:
+	AudioFileCopyConvert* m_converter;
+	ExportSpecification* m_exportSpec;
+	ExportFormatOptionsWidget* m_formatOptionsWidget;
+	QButtonGroup* m_buttonGroup;
+
+	void load_all_files();
+	void copy_files();
+
 private slots:
 	void accept();
 	void use_template_checkbox_state_changed(int state);
 	void update_template_combobox();
+	void add_files();
+	void remove_files();
+	void load_file(QString, int, QString);
+	void move_up();
+	void move_down();
+
+signals:
+	void loadFile(QString, int, QString);
+	void number_of_files(int);
 };
 
 #endif
